<?php

/**
 * Sync
 */

namespace NMGR\Events;

defined( 'ABSPATH' ) || exit;

use NmeriMedia\V2\Scheduler;

/**
 * Delete guest wishlists if they have expired based on the registered cookie expiry time
 *
 * @access private
 */
class DeleteGuestWishlists extends Scheduler {

	protected $recurrence = true;
	protected $category;

	public function __construct( $category ) {
		$this->category = $category;
		$this->prefix = 'nmgr_' . $this->category;
		$this->timestamp = time() + (2 * DAY_IN_SECONDS);
		parent::__construct();
	}

	protected function task( $id ) {
		$wishlist = nmgr_get_wishlist( $id );
		if ( $wishlist ) {
			$wishlist->delete();
		}
		return false;
	}

	protected function get_batch_data() {
		return $this->get_wishlist_ids();
	}

	/**
	 *
	 * @global type $wpdb
	 * @return array Database query result
	 */
	private function get_wishlist_ids() {
		global $wpdb;

		$cache_key = 'nmgr_guest_wishlist_ids_' . $this->category;
		$ids = wp_cache_get( $cache_key );

		if ( false === $ids ) {
			$ids = [];

			$post_ids = $wpdb->get_col( $wpdb->prepare( "
				SELECT ID FROM {$wpdb->posts} posts
		INNER JOIN {$wpdb->term_relationships} term_relationships
		ON posts.ID = term_relationships.object_id
		INNER JOIN {$wpdb->term_taxonomy} term_taxonomy
		ON term_taxonomy.term_taxonomy_id = term_relationships.term_taxonomy_id
		INNER JOIN {$wpdb->terms} terms
		ON terms.term_id = term_taxonomy.term_taxonomy_id
					WHERE post_author = 0
					AND post_type = 'nm_gift_registry'
					AND post_status != 'trash'
					AND term_taxonomy.taxonomy = 'nm_gift_registry_type'
		AND terms.slug = %s
				 ",
					$this->category
				) );

			foreach ( $post_ids as $id ) {
				$post = get_post( $id );
				$creation_date = strtotime( $post->post_date );
				$expiry_days = nmgr_get_guest_wishlist_expiry_days();
				$expiry_date = strtotime( $expiry_days . ' DAYS', $creation_date );
				if ( $expiry_date < time() ) {
					$ids[] = $id;
				}
			}

			wp_cache_set( $cache_key, $ids );
		}

		return $ids;
	}

}
