<?php
/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

class NMGR_Admin_Settings extends NmeriMedia\V2\Settings {

	public $option_name = 'nmgr_settings';
	public $is_woocommerce_screen = true;

	public function __construct( $plugin_fn_name = null ) {
		parent::__construct( $plugin_fn_name );
	}

	public function get_type() {
		return 'gift-registry';
	}

	/**
	 * For backwards compatibility with plugins using this method
	 * e.g. crowdfunding extension.
	 * @since version 3.0.0
	 * Remove in a much later version
	 */
	public static function get_default_values() {
		return function_exists( 'nmgr' ) ? nmgr()->gift_registry_settings()->get_default_field_values() : [];
	}

	public function is_nmerimedia_screen() {
		global $typenow, $pagenow;

		return parent::is_nmerimedia_screen() ||
			(function_exists( 'is_nmgr_admin' ) && is_nmgr_admin()) ||
			('product' === $typenow && 'edit.php' === $pagenow);
	}

	public function init() {
		parent::init();

		add_action( $this->page_slug() . '_tab_' . $this->current_tab, [ $this, 'current_section_tab_action' ] );
		add_filter( $this->page_slug() . '_tab_sections', [ $this, 'tab_sections_filter' ], 0, 2 );
		add_filter( $this->page_slug() . '_tab_title', [ $this, 'tab_title_filter' ], 10, 3 );
		add_filter( $this->page_slug() . '_tab_title', [ $this, 'modify_tab_title' ], 10, 2 );
		add_filter( $this->page_slug() . '_tab_title', [ $this, 'show_template_overrides_notice' ], 10, 2 );
		add_filter( $this->page_slug() . '_validate_input', [ $this, 'extra_validate_fields_to_save' ] );
		add_action( $this->page_slug() . '_tab_overrides', [ $this, 'do_settings_sections_overrides' ] );
		add_action( $this->page_slug() . '_tab_full_version', [ $this, 'do_settings_sections_full_version' ] );
		add_action( $this->page_slug() . '_before_save_input', [ $this, 'save_settings_errors' ] );
		add_action( 'admin_init', [ $this, 'maybe_flush_rewrite_rules' ], 999 );
	}

	public function maybe_flush_rewrite_rules() {
		if ( get_transient( 'nmgr_flush_rewrite_rules' ) ) {
			delete_transient( 'nmgr_flush_rewrite_rules' );
			flush_rewrite_rules();
		}
	}

	public function current_section_tab_action() {
		do_action_deprecated( 'nmgr_settings_sections_tab_' . $this->current_tab,
			[],
			'3.0.0',
			$this->page_slug() . '_tab_' . $this->current_tab
		);
	}

	public function tab_sections_filter( $section, $tab ) {
		return apply_filters_deprecated( 'nmgr_settings_tab_sections',
			[ $section, $tab ],
			'3.0.0',
			$this->page_slug() . '_tab_sections'
		);
	}

	public function tab_title_filter( $title, $slug, $class ) {
		return apply_filters_deprecated( 'nmgr_settings_tab_title',
			[ $title, $class ],
			'3.0.0',
			$this->page_slug() . '_tab_title'
		);
	}

	public function modify_tab_title( $title, $slug ) {
		if ( 'emails' === $slug && !nmgr()->is_pro ) {
			$title .= ' ' . $this->get_pro_version_text();
		}
		return $title;
	}

	public function get_screen_id() {
		return 'nm_gift_registry_page_' . $this->page_slug();
	}

	public function add_menu_page() {
		$title = $this->get_heading();

		add_submenu_page(
			'edit.php?post_type=nm_gift_registry',
			$title . ' - ' . nmgr()->name,
			$title,
			'manage_nm_gift_registry_settings',
			$this->page_slug(),
			array( $this, 'menu_page_content' )
		);
	}

	public function get_custom_tabs() {
		return [];
	}

	public function get_tabs() {
		$tabs = array_merge( $this->get_custom_tabs(), [
			'overrides' => array(
				'tab_title' => nmgr()->is_pro ?
				__( 'Overrides', 'nm-gift-registry' ) :
				__( 'Overrides', 'nm-gift-registry-lite' ),
				'sections_title' => '',
				'show_sections' => false,
				'submit_button' => false,
			),
			] );

		if ( !nmgr()->is_pro ) {
			$tabs[ 'full_version' ] = array(
				'tab_title' => __( 'PRO Version', 'nm-gift-registry-lite' ),
				'sections_title' => '',
				'show_sections' => false,
				'submit_button' => false
			);
		}

		return apply_filters( 'nmgr_settings_tabs', $tabs, $this->get_type() );
	}

	public function do_settings_sections_full_version() {
		if ( nmgr()->is_pro ) {
			return;
		}

		$features = array(
			array(
				'title' => __( 'Multiple wishlists', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow users to have as many gift registries or wishlists as they want, each with it\'s own custom management page and settings on the frontend.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'database',
					'size' => 6,
					'fill' => '#da5027',
					'sprite' => false
				) ),
			),
			array(
				'title' => __( 'Wishlist lifecycle emails', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow the plugin to send emails to custom recipients and the list owner when the list is created, fulfilled or deleted, when items have been ordered, purchased or refunded from the list, and when messages are sent to the list owner at checkout.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'email',
					'size' => 6,
					'fill' => 'blue'
				) ),
			),
			array(
				'title' => __( 'Featured and background images', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow list owners to upload and display featured and background images on their list pages and on social media to add a personalised touch and improve the impressions made on guests.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'camera',
					'size' => 6,
					'fill' => 'lawngreen'
				) ),
			),
			array(
				'title' => __( 'Checkout messages', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow guests to send messages to the list owner from the checkout page as items are ordered for him. These message appear in the list\'s management page and can be configured to appear in the owner\'s inbox as well as in other emails sent to him.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'bubble',
					'size' => 6,
					'fill' => 'aqua'
				) ),
			),
			array(
				'title' => __( 'Wishlist visibility settings', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow list owners to set the visibility of their lists to private, password protected or public from the list management page on the frontend. These visibilities correspond to WordPress\' visibility settings.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'gear',
					'size' => 6,
					'fill' => 'purple'
				) ),
			),
			array(
				'title' => __( 'Exclude wishlists from search', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow list owners to exclude individual lists from search results regardless of their visibility. This gives them improved control over how their lists appear on the website.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'search',
					'size' => 6,
					'sprite' => false,
					'fill' => 'yellow'
				) ),
			),
			array(
				'title' => __( 'Mark wishlist items as favourite', 'nm-gift-registry-lite' ),
				'desc' => __( 'Allow list owners to mark an item as favourite when adding it to their list. The favourite status of items can be edited in the list management page and items can be sorted in the items table by their favourite status.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'star-full',
					'size' => 6,
					'fill' => 'red'
				) ),
			),
			array(
				'title' => __( 'Extra add to wishlist customizations', 'nm-gift-registry-lite' ),
				'desc' => __( 'Customize the add to wishlist button to your liking. Select new display types, toggle the use of ajax for the action, the animation and display of notifications, the ability to choose the quantity and favourite status when adding a product to the list, whether variable and grouped products can be added to the list. Include or exclude products and product categories from being added to the wishlist, and much more.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'wrench',
					'size' => 6,
					'sprite' => false,
					'fill' => 'orange'
				) ),
			),
			array(
				'title' => __( 'One-click wishlist templates customization', 'nm-gift-registry-lite' ),
				'desc' => __( 'Customize the list templates from the admin area with the click of a button without writing any code. Toggle the visibility of default account tabs, the visibility and required status of the settings fields, the visibility and display type of the featured and background images, the messages module, and much more.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'control-panel',
					'size' => 6,
					'sprite' => false,
					'fill' => '#1d78c3'
				) ),
			),
			array(
				'title' => __( 'Delete wishlist', 'nm-gift-registry-lite' ),
				'desc' => __( 'Give list owners the ability to delete their lists from the frontend without having to contact the admin or leave it dormant to clog up your database. This gives them more control over their lists and leads to a cleaner site for the administrator.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'trash-can',
					'size' => 6,
					'fill' => 'brown'
				) ),
			),
			array(
				'title' => __( 'Customize wishlist shipping', 'nm-gift-registry-lite' ),
				'desc' => __( 'Control the shipping methods available for wishlist items in the cart. Calculate shipping rates for the wishlist items separately from normal items. Ship wishlist items in cart to the wishlist\'s owner\'s address. And much more.', 'nm-gift-registry-lite' ),
				'image' => nmgr_get_svg( array(
					'icon' => 'box-open',
					'size' => 6,
					'fill' => 'orangered'
				) ),
			),
		);
		?>
		<div class="nmgr-full-version">
			<div class="nmgr-text-center">
				<a class="nmgr-buy-btn" href="<?php echo esc_url( nmgr()->product_url ); ?>" rel="noopener noreferrer" target="_blank"><?php esc_html_e( 'Upgrade Now', 'nm-gift-registry-lite' ); ?></a>
			</div>

			<h1 class="nmgr-text-center"><?php esc_html_e( 'NM Gift Registry and Wishlist Features', 'nm-gift-registry-lite' ); ?></h1>
			<p class="nmgr-desc nmgr-text-center"><?php esc_html_e( 'Check out these fantastic extra features that can provide your store with the perfect gift registry and wishlist experience.', 'nm-gift-registry-lite' ); ?></p>
			<div class="nmgr-features">
				<?php foreach ( $features as $feature ) : ?>
					<div class="nmgr-feature">
						<div class="nmgr-image"><?php echo $feature[ 'image' ]; ?></div>
						<div class="nmgr-info">
							<h2><?php echo esc_html( $feature[ 'title' ] ); ?></h2>
							<p><?php echo wp_kses_post( $feature[ 'desc' ] ); ?></p>
						</div>
					</div>
				<?php endforeach; ?>
			</div>

			<div class="nmgr-text-center">
				<a class="nmgr-buy-btn" href="<?php echo esc_url( nmgr()->product_url ); ?>" rel="noopener noreferrer" target="_blank"><?php esc_html_e( 'Upgrade Now', 'nm-gift-registry-lite' ); ?></a>
			</div>
		</div>
		<?php
	}

	public function get_theme_overrides_info() {
		if ( !class_exists( '\WC_Admin_Status' ) ||
			!method_exists( '\WC_Admin_Status', 'scan_template_files' ) ||
			!method_exists( '\WC_Admin_Status', 'get_file_version' ) ) {
			return [];
		}

		/**
		 * Scan the theme directory for all WC templates to see if our theme
		 * overrides any of them.
		 */
		$override_files = array();
		$outdated_templates = false;
		$scan_files = \WC_Admin_Status::scan_template_files( nmgr()->template_path() );

		foreach ( $scan_files as $file ) {
			$located = apply_filters( 'wc_get_template', $file, $file, array(), nmgr()->theme_path(), nmgr()->template_path() );

			if ( file_exists( $located ) ) {
				$theme_file = $located;
			} elseif ( file_exists( get_stylesheet_directory() . '/' . $file ) ) {
				$theme_file = get_stylesheet_directory() . '/' . $file;
			} elseif ( file_exists( get_stylesheet_directory() . '/' . nmgr()->theme_path() . $file ) ) {
				$theme_file = get_stylesheet_directory() . '/' . nmgr()->theme_path() . $file;
			} elseif ( file_exists( get_template_directory() . '/' . $file ) ) {
				$theme_file = get_template_directory() . '/' . $file;
			} elseif ( file_exists( get_template_directory() . '/' . nmgr()->theme_path() . $file ) ) {
				$theme_file = get_template_directory() . '/' . nmgr()->theme_path() . $file;
			} else {
				$theme_file = false;
			}

			if ( !empty( $theme_file ) ) {
				$core_file = $file;
				$core_version = \WC_Admin_Status::get_file_version( nmgr()->template_path() . $core_file );
				$theme_version = \WC_Admin_Status::get_file_version( $theme_file );
				if ( $core_version && ( empty( $theme_version ) || version_compare( $theme_version, $core_version, '<' ) ) ) {
					if ( !$outdated_templates ) {
						$outdated_templates = true;
					}
				}
				$override_files[] = array(
					'file' => str_replace( WP_CONTENT_DIR . '/themes/', '', $theme_file ),
					'version' => $theme_version,
					'core_version' => $core_version,
				);
			}
		}

		$active_theme = wp_get_theme();
		return array(
			'name' => $active_theme->name,
			'version' => $active_theme->version,
			'is_child_theme' => is_child_theme(),
			'has_outdated_templates' => $outdated_templates,
			'overrides' => $override_files,
		);
	}

	public function do_settings_sections_overrides() {
		$toi = $this->get_theme_overrides_info();

		if ( empty( $toi ) ) {
			return;
		}

		echo "<h2>" . esc_html( nmgr()->is_pro ?
				__( 'This section shows files in your current theme that are overriding NM Gift Registry template files.', 'nm-gift-registry' ) :
				__( 'This section shows files in your current theme that are overriding NM Gift Registry template files.', 'nm-gift-registry-lite' )
		) . "</h2>";

		echo '<p><strong>' . sprintf(
			/* translators: %s: Current theme */
			esc_html( nmgr()->is_pro ? __( 'Current theme: %s', 'nm-gift-registry' ) : __( 'Current theme: %s', 'nm-gift-registry-lite' )  ),
			$toi[ 'name' ]
		) . '</strong></p>';

		if ( empty( $toi[ 'overrides' ] ) ) {
			echo '<p>' . esc_html( nmgr()->is_pro ?
					__( 'You are not currently overriding any NM Gift Registry template files.', 'nm-gift-registry' ) :
					__( 'You are not currently overriding any NM Gift Registry template files.', 'nm-gift-registry-lite' )
			) . '</p>';
			return;
		}
		?>
		<table class="widefat">
			<thead><th colspan="2"><?php
				echo esc_html( nmgr()->is_pro ?
						__( 'Template overrides', 'nm-gift-registry' ) :
						__( 'Template overrides', 'nm-gift-registry-lite' )
				);
				?></th></thead>
		<tbody>
			<?php foreach ( $toi[ 'overrides' ] as $override ) : ?>
				<tr>
					<td><?php echo '<code>' . esc_html( $override[ 'file' ] ) . '</code>'; ?></td>
					<td>
						<?php
						if ( $override[ 'core_version' ] && ( empty( $override[ 'version' ] ) ||
							version_compare( $override[ 'version' ], $override[ 'core_version' ], '<' ) ) ) {
							$current_version = $override[ 'version' ] ? $override[ 'version' ] : '-';
							printf(
								/* Translators: %1$s: Template version, %2$s: Core version. */
								esc_html( nmgr()->is_pro ? __( 'Version %1$s is out of date. The core version is %2$s', 'nm-gift-registry' ) : __( 'Version %1$s is out of date. The core version is %2$s', 'nm-gift-registry-lite' )  ),
								'<strong style="color:red">' . esc_html( $current_version ) . '</strong>',
								esc_html( $override[ 'core_version' ] )
							);
						}
						?>
					</td>
				</tr>
			<?php endforeach; ?>
		</tbody>
		</table>
		<?php
	}

	public function show_template_overrides_notice( $title, $slug ) {
		if ( 'overrides' === $slug ) {
			$toi = $this->get_theme_overrides_info();
			if ( !empty( $toi ) && true === $toi[ 'has_outdated_templates' ] ) {
				$title .= nmgr_get_svg( array(
					'icon' => 'info',
					'style' => 'margin-left:10px;',
					'class' => 'align-with-text nmgr-tip',
					'title' => nmgr()->is_pro ?
					__( 'You have some outdated overriding templates', 'nm-gift-registry' ) :
					__( 'You have some outdated overriding templates', 'nm-gift-registry-lite' ),
					'fill' => 'red',
					) );
			}
		}
		return $title;
	}

	/**
	 *  Validate custom fields that have not been captured above in the standard tab section fields
	 * @todo make this process more dynamic in the future
	 */
	public function extra_validate_fields_to_save( $input ) {
		return $input;
	}

	public function get_sidebar_contents() {
		$notice = '';

		if ( !nmgr()->is_pro ) {
			$notice = $this->get_buy_pro_notice();

			if ( $notice ) {
				// Style like admin notice info
				$style = 'background:white;padding:12px;border:1px solid #c3c4c7;border-left:4px solid #72aee6;';
				$notice = '<div class="notice-info" style="' . $style . '">' . $notice . '</div>';
			}
		}

		return parent::get_sidebar_contents() . $notice;
	}

	public function get_error_codes_to_messages() {
		$no_page_id = sprintf(
			/* translators: %s: wishlist type title */
			nmgr()->is_pro ? __( 'You need to set a page for viewing %s.', 'nm-gift-registry' ) : __( 'You need to set a page for viewing %s.', 'nm-gift-registry-lite' ),
			nmgr_get_type_title( '', 1, $this->get_type() )
		);

		$msgs = [
			'no-page-id' => $no_page_id,
			'no-wishlist-page-id' => $no_page_id,
		];

		return $msgs;
	}

	public function save_settings_errors( $input ) {
		$options = array_merge( nmgr_get_option(), $input );

		$params = [
			[
				'enable' => 'enable',
				'page_id' => 'page_id',
				'code' => 'no-page-id',
			],
			[
				'enable' => 'wishlist_enable',
				'page_id' => 'wishlist_page_id',
				'code' => 'no-wishlist-page-id',
			],
		];

		foreach ( $params as $param ) {
			if ( array_key_exists( $param[ 'page_id' ], $input ) && empty( $input[ $param[ 'page_id' ] ] ) &&
				isset( $options[ $param[ 'enable' ] ] ) && $options[ $param[ 'enable' ] ] ) {
				$code = $param[ 'code' ];
				$this->save_settings_error( $code, $this->get_error_message_by_code( $code ) );
			}
		}
	}

	public function get_onsale_plugins() {
		return '';
	}

	public function get_default_field_values() {
		$defaults = parent::get_default_field_values();

		if ( !nmgr()->is_pro ) {
			$pro_defaults = $this->get_default_pro_fields_values();
			$defaults = array_diff_key( $defaults, $pro_defaults );
		}

		return $defaults;
	}

	public function get_default_pro_fields_values() {
		$default_pro_fields = [];

		foreach ( $this->get_fields() as $key => $field ) {
			if ( !empty( $field[ 'pro' ] ) ) {
				$default_pro_fields[ $key ] = $field;
			}
		}

		return $this->get_default_values_for_fields( $default_pro_fields );
	}

}
