<?php

/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

use NMGR\Events\DeleteGuestWishlists;
use NMGR\Events\DeleteExtraUserWishlists;
use NMGR\Events\SetExpiredWishlists;
use NMGR\Events\SetWishlistTerms;

/**
 * Incomplete class
 * Do not use
 * @access private
 * @since version 3.0.0
 */
class NMGR_Events {

	protected static $events = [];

	public static function run() {
		register_deactivation_hook( nmgr()->file, [ __CLASS__, 'clear_autoscheduled_events' ] );
		add_action( 'init', [ __CLASS__, 'initialize_events' ], 10 );
	}

	/**
	 * @return \NmeriMedia\V2\Scheduler[]
	 */
	public static function get_events( $event = null ) {
		return $event ? self::$events[ $event ] : self::$events;
	}

	public static function initialize_events() {
		self::$events[ 'delete_guest_gift_registries' ] = new DeleteGuestWishlists( 'gift-registry' );
		self::$events[ 'delete_guest_wishlists' ] = new DeleteGuestWishlists( 'wishlist' );
		self::$events[ 'delete_extra_user_gift_registries' ] = new DeleteExtraUserWishlists( 'gift-registry' );
		self::$events[ 'delete_extra_user_wishlists' ] = new DeleteExtraUserWishlists( 'wishlist' );
		self::$events[ 'set_expired_gift_registries' ] = new SetExpiredWishlists();
		self::$events[ 'set_wishlist_terms' ] = new SetWishlistTerms();

		foreach ( self::get_events() as $event ) {
			// Run only events that are scheduled to recur
			if ( method_exists( $event, 'recurs' ) && $event->recurs() && method_exists( $event, 'run' ) ) {
				$event->run();
			}
		}
	}

	public static function clear_autoscheduled_events() {
		foreach ( self::get_events() as $event ) {
			// Run only events that are scheduled to recur
			if ( method_exists( $event, 'recurs' ) && $event->recurs() && method_exists( $event, 'unschedule' ) ) {
				$event->unschedule();
			}
		}
	}

	public static function delete_extra_user_wishlists() {
		self::get_events( 'delete_extra_user_wishlists' )->run();
	}

	public static function delete_extra_user_gift_registries() {
		self::get_events( 'delete_extra_user_gift_registries' )->run();
	}

	public static function set_wishlist_terms() {
		self::get_events( 'set_wishlist_terms' )->run();
	}

}
