<?php
/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

class NMGR_Order {

	public static function run() {
		add_filter( 'woocommerce_continue_shopping_redirect', array( __CLASS__, 'continue_shopping_redirect' ) );
		add_filter( 'woocommerce_add_cart_item_data', array( __CLASS__, 'add_wishlist_item_cart_item_data' ), 10, 2 );
		add_filter( 'woocommerce_add_to_cart_validation', array( __CLASS__, 'maybe_add_wishlist_item_to_cart' ), 10, 3 );
		add_filter( 'woocommerce_update_cart_validation', array( __CLASS__, 'maybe_update_cart_item_quantity' ), 10, 4 );
		add_action( 'woocommerce_before_cart', array( __CLASS__, 'notify_if_cart_has_wishlist' ) );
		add_filter( 'woocommerce_get_item_data', array( __CLASS__, 'show_wishlist_item_cart_item_data' ), 50, 2 );
		add_action( 'woocommerce_before_checkout_shipping_form', array( __CLASS__, 'notify_wishlist_shipping' ) );
		add_action( 'woocommerce_checkout_create_order_line_item', array( __CLASS__, 'add_wishlist_data_as_order_item_meta_data' ), 10, 3 );
		add_action( 'woocommerce_checkout_order_created', [ __CLASS__, 'add_wishlist_data_as_order_meta_data' ] );
		add_action( 'nmgr_created_order', [ __CLASS__, 'do_process_wishlist_item_payment' ] );
		add_action( 'woocommerce_pre_payment_complete', [ __CLASS__, 'do_process_wishlist_item_payment' ] );
		add_action( 'woocommerce_payment_complete', array( __CLASS__, 'process_wishlist_item_payment' ), 10 );
		add_action( 'woocommerce_order_status_changed', array( __CLASS__, 'process_wishlist_item_payment' ), 10 );
		add_action( 'woocommerce_order_refunded', array( __CLASS__, 'process_wishlist_item_payment' ), 10 );
		add_action( 'woocommerce_refund_deleted', array( __CLASS__, 'process_wishlist_item_payment_for_deleted_refunds' ), 10, 2 );
		add_action( 'nmgr_order_payment_complete', array( __CLASS__, 'update_wishlist_item_purchased_quantity' ), 10 );
		add_action( 'nmgr_order_payment_complete', array( __CLASS__, 'maybe_set_wishlist_as_fulfilled' ), 99 );
		add_action( 'woocommerce_check_cart_items', array( __CLASS__, 'check_cart_items_for_shipping_address' ) );
		add_filter( 'woocommerce_order_item_get_formatted_meta_data', [ __CLASS__, 'show_order_item_meta_data' ], 10, 2 );
		add_action( 'woocommerce_before_single_product', [ __CLASS__, 'add_wishlist_product_to_cart_from_product_page' ] );
		add_action( 'admin_notices', [ __CLASS__, 'show_custom_order_notice' ] );
		add_action( 'woocommerce_email_customer_details', [ __CLASS__, 'show_custom_order_notice_in_email' ], 30, 3 );
	}

	/**
	 * If the product was added to the cart from the wishlist page, return to the wishlist page
	 */
	public static function continue_shopping_redirect( $url ) {
		$item_data = self::get_add_to_cart_item_data();

		if ( empty( $item_data[ 'nmgr-add-to-cart-wishlist' ] ) ) {
			return $url;
		}

		$wishlist = nmgr_get_wishlist( $item_data[ 'nmgr-add-to-cart-wishlist' ] );
		return $wishlist ? $wishlist->get_permalink() : $url;
	}

	/**
	 * Get the data of the wishlist items we are adding to the cart from the request array
	 *
	 * This is necessary as the wishlist items can be added to the cart individually or in bulk.
	 * In each of these cases, the request array is different. But we need to make it consistent
	 * in order to apply the same functions to them. Hence this function.
	 * @return array
	 */
	public static function get_add_to_cart_item_data( $product_id = 0 ) {
		$item_data = array();

		if ( isset( $_REQUEST[ 'nmgr_add_to_cart_item_data' ] ) ) {
			$bulk_data = $_REQUEST[ 'nmgr_add_to_cart_item_data' ];
		} elseif ( isset( $_REQUEST[ 'nmgr_add_to_cart_item_data_string' ] ) ) {
			$bulk_data = json_decode( stripslashes( $_REQUEST[ 'nmgr_add_to_cart_item_data_string' ] ) );
		}

		if ( isset( $_REQUEST[ 'nmgr-add-to-cart-wishlist' ] ) ) {
			$item_data = $_REQUEST;
		} elseif ( isset( $bulk_data ) ) {
			foreach ( $bulk_data as $item_mixed ) {
				/**
				 * Cast to array as $item_mixed might be an object depending on the source
				 */
				$item = ( array ) $item_mixed;
				if ( $product_id && isset( $item[ 'nmgr-add-to-cart-product-id' ] ) && ( int ) $product_id === ( int ) $item[ 'nmgr-add-to-cart-product-id' ] ) {
					$item_data = $item;
					break;
				}
			}

			if ( empty( $item_data ) && isset( $_REQUEST[ 'items' ] ) ) {
				$item_data = reset( $_REQUEST[ 'items' ] );
			}
		}

		return apply_filters( 'nmgr_get_add_to_cart_item_data', $item_data, $product_id );
	}

	/**
	 * Add information about the wishlist to the cart item when it is added to the cart
	 */
	public static function add_wishlist_item_cart_item_data( $cart_item_data, $product_id ) {
		$item_data = self::get_add_to_cart_item_data( $product_id );

		if ( !empty( $item_data[ 'nmgr-add-to-cart-wishlist-item' ] ) ) {
			$wishlist_item = nmgr_get_wishlist_item( $item_data[ 'nmgr-add-to-cart-wishlist-item' ] );

			if ( !empty( $wishlist_item ) && $wishlist_item->get_wishlist()->is_type( 'gift-registry' ) ) {
				$cart_item_data[ 'nm_gift_registry' ] = $wishlist_item->get_cart_order_data();
			}
		}

		return $cart_item_data;
	}

	/**
	 * Add the wishlist item to the cart if the quantity being added is not more than the quantity requested in the wishlist
	 * This function is used typically on the wishlist page where a wishlist item is being added to the cart
	 *
	 * @param bool $passed Whether the validation passes or fails
	 * @param int $product_id Item product id
	 * @param int $quantity Item quantity being added to cart
	 * @return boolean Validation passed or failed
	 */
	public static function maybe_add_wishlist_item_to_cart( $passed, $product_id, $quantity ) {
		$item_data = self::get_add_to_cart_item_data( $product_id );

		if ( empty( $item_data ) ) {
			return $passed;
		}

		$wishlist_id = ( int ) $item_data[ 'nmgr-add-to-cart-wishlist' ];
		$wishlist_item_id = ( int ) $item_data[ 'nmgr-add-to-cart-wishlist-item' ];

		// check if the wishlist item is already in the cart
		$item_in_cart = false;
		foreach ( WC()->cart->get_cart() as $cart_item ) {
			$nmgr_data = nmgr_get_cart_item_data( $cart_item, 'wishlist_item' );
			if ( $nmgr_data ) {
				if ( ( int ) $nmgr_data[ 'wishlist_id' ] === $wishlist_id &&
					( int ) $nmgr_data[ 'wishlist_item_id' ] === $wishlist_item_id ) {
					$item_in_cart = $cart_item;
					break;
				}
			}
		}

		return self::validate_wishlist_item_cart_quantity( $passed, $wishlist_id, $wishlist_item_id, $quantity, $item_in_cart );
	}

	/**
	 * Update the wishlist item in the cart if the quantity being added doesn't exceed the quantity requested in the wishlist
	 * This function is used mainly on the cart page where only the item quantity is updated
	 *
	 * @param bool $passed Whether the validation passes or fails
	 * @param string $cart_item_key Unique id of the cart item
	 * @param array $values Cart item properties
	 * @param int $quantity Item quantity being added to cart
	 * @return boolean Validation passed or failed
	 */
	public static function maybe_update_cart_item_quantity( $passed, $cart_item_key, $values, $quantity ) {
		$passed = self::validate_product_is_wishlist_item_cart_quantity( $passed, $values, $quantity );
		$nmgr_data = nmgr_get_cart_item_data( $values, 'wishlist_item' );

		if ( $passed && $nmgr_data ) {
			$passed = self::validate_wishlist_item_cart_quantity( $passed, $nmgr_data[ 'wishlist_id' ], $nmgr_data[ 'wishlist_item_id' ], $quantity );
		}

		return $passed;
	}

	/**
	 * Validate the quantity of a wishlist item added to the cart based on its existing cart quantity if present and desired quantity
	 *
	 * @param bool $passed Whether the validation passes or fails
	 * @param int $wishlist_id The wishlist id
	 * @param int $wishlist_item_id The item id in the wishlist
	 * @param int $quantity The quantity of the item being added
	 * @param array $item_in_cart The item contents if the item is already in the cart
	 * @return boolean Validation passed or failed
	 */
	private static function validate_wishlist_item_cart_quantity( $passed, $wishlist_id, $wishlist_item_id, $quantity, $item_in_cart = false ) {
		$wishlist = nmgr_get_wishlist( $wishlist_id, true );
		if ( !$wishlist || $wishlist->is_type( 'wishlist' ) ||
			!nmgr_get_type_option( $wishlist->get_type(), 'display_item_quantity' ) ) {
			return $passed;
		}

		$item = $wishlist->get_item( $wishlist_item_id );
		if ( $item ) {
			$item_title = sprintf(
				/* translators: %s: item name */
				nmgr()->is_pro ? _x( '&ldquo;%s&rdquo;', 'Item name in quotes', 'nm-gift-registry' ) : _x( '&ldquo;%s&rdquo;', 'Item name in quotes', 'nm-gift-registry-lite' ),
				$item->get_product()->get_name()
			);

			$desired_item_qty = nmgr_get_type_option( $wishlist->get_type(), 'display_item_purchased_quantity' ) ?
				$item->get_unpurchased_quantity() : $item->get_quantity();

			if ( $item_in_cart ) {
				$item_cart_quantity = $item_in_cart[ 'quantity' ]; // the current quantity of the item in the cart

				if ( ($quantity + $item_cart_quantity) > $desired_item_qty ) {
					$passed = false;
					$message = apply_filters( 'nmgr_validate_wishlist_item_in_cart_quantity_message', sprintf(
							/* translators:
							 * 1: item quantity to add to cart,
							 * 2: item title,
							 * 3: item quantity in cart,
							 * 4: item quantity requested in wishlist,
							 * 5: wishlist type title
							 */
							nmgr()->is_pro ? __( 'You cannot add %1$d of %2$s to the cart as you have %3$d already in the cart with %4$d requested in the %5$s.', 'nm-gift-registry' ) : __( 'You cannot add %1$d of %2$s to the cart as you have %3$d already in the cart with %4$d requested in the %5$s.', 'nm-gift-registry-lite' ),
							$quantity,
							$item_title,
							$item_cart_quantity,
							$desired_item_qty,
							nmgr_get_type_title( '', false, $wishlist->get_type() )
						), $quantity, $item, $wishlist, $item_in_cart );
					wc_add_notice( $message, 'error' );
				}
			} else {
				if ( $quantity > $desired_item_qty ) {
					$passed = false;
					$message = apply_filters( 'nmgr_validate_wishlist_item_cart_quantity_message', sprintf(
							/* translators: 1: item title, 2: wishlist type title */
							nmgr()->is_pro ? __( 'Please choose a quantity of %1$s that is not greater than the quantity requested in the %2$s.', 'nm-gift-registry' ) : __( 'Please choose a quantity of %1$s that is not greater than the quantity requested in the %2$s.', 'nm-gift-registry-lite' ),
							$item_title,
							nmgr_get_type_title( '', false, $wishlist->get_type() )
						), $quantity, $item, $wishlist );
					wc_add_notice( $message, 'error' );
				}
			}
		}
		return $passed;
	}

	/**
	 * Validate the quantity of items added to the cart if these items have already been
	 * added to the cart as wishlist items
	 *
	 * @param bool $passed Whether the validation passes or fails
	 * @param int $values Cart item data
	 * @param int $quantity The quantity of the item being added
	 * @return boolean Validation passed or failed
	 */
	public static function validate_product_is_wishlist_item_cart_quantity( $passed, $values, $quantity ) {
		$product = $values[ 'data' ];

		if ( !$product->managing_stock() || $product->backorders_allowed() ) {
			return $passed;
		}

		// check if product is also added as a wishlist item
		$product_is_wishlist_item_in_cart = false;

		foreach ( WC()->cart->get_cart() as $cart_item ) {
			$nmgr_data = nmgr_get_cart_item_data( $cart_item, 'wishlist_item' );
			if ( $nmgr_data ) {
				$variation_id = $nmgr_data[ 'variation_id' ] ?? 0;
				if ( ($variation_id &&
					$variation_id == $values[ 'variation_id' ]) ||
					$nmgr_data[ 'product_id' ] == $values[ 'product_id' ] ) {
					$product_is_wishlist_item_in_cart = true;
					break;
				}
			}
		}

		if ( !$product_is_wishlist_item_in_cart ) {
			return $passed;
		}

		// Check that the overall cart stock for the product is not greater than the product stock
		$products_cart_quantities = wc()->cart->get_cart_item_quantities();
		$product_cart_quantity = $products_cart_quantities[ $product->get_stock_managed_by_id() ];

		if ( $product->get_stock_quantity() < ($product_cart_quantity + $quantity) ) {
			$message = sprintf(
				/* translators: %1$s: product stock quantity, %2$s: product quantity in cart */
				nmgr()->is_pro ? __( 'You cannot add that amount to the cart &mdash; we have %1$s in stock and you already have %2$s in your cart.', 'nm-gift-registry' ) : __( 'You cannot add that amount to the cart &mdash; we have %1$s in stock and you already have %2$s in your cart.', 'nm-gift-registry-lite' ),
				$product->get_stock_quantity(),
				$product_cart_quantity
			);
			wc_add_notice( $message, 'error' );
			return false;
		}
		return $passed;
	}

	/**
	 * Add a simple notice on the cart page to notify that the cart contains at least one wishlist
	 */
	public static function notify_if_cart_has_wishlist() {
		$cart_has_wishlist = nmgr_get_wishlist_in_cart();
		$message = '';

		if ( nmgr()->is_pro &&
			function_exists( 'is_nmgr_cart_shipping_to_wishlist_address' ) &&
			is_nmgr_cart_shipping_to_wishlist_address() ) {
			$message = sprintf(
				/* translators: 1, 2: wishlist type title */
				__( 'You have %1$s items in your cart. Please note that all items in the cart are shipping to the address of the %s owner.', 'nm-gift-registry' ),
				nmgr_get_type_title(),
				nmgr_get_type_title()
			);
		} elseif ( nmgr()->is_pro && $cart_has_wishlist && nmgr_get_option( 'shipping_calculate' ) ) {
			$message = sprintf(
				/* translators: %s: wishlist type title */
				__( 'You have %s items in your cart. Please note that shipping for these items is set up separately.', 'nm-gift-registry' ),
				nmgr_get_type_title()
			);
		} elseif ( $cart_has_wishlist ) {
			$message = sprintf(
				/* translators: 1, 2: wishlist type title */
				nmgr()->is_pro ? __( 'You have %1$s items in your cart.', 'nm-gift-registry' ) : __( 'You have %1$s items in your cart.', 'nm-gift-registry-lite' ),
				nmgr_get_type_title()
			);
		}

		$notice = apply_filters( 'nmgr_cart_has_wishlist_notice', $message );

		if ( $notice ) {
			wc_print_notice( $notice, 'notice' );
		}
	}

	/**
	 * Display information about the associated wishlist for an item in the cart
	 *
	 * @param array $item_data key value pair of Information to display
	 * @param array $cart_item_data Cart item data
	 * @return array $item_data
	 */
	public static function show_wishlist_item_cart_item_data( $item_data, $cart_item_data ) {
		$nmgr_data = nmgr_get_cart_item_data( $cart_item_data, 'wishlist_item' );

		if ( !$nmgr_data || empty( $nmgr_data ) || empty( array_filter( $nmgr_data ) ) ) {
			return $item_data;
		}

		$wishlist = nmgr_get_wishlist( $nmgr_data[ 'wishlist_id' ], true );

		if ( !$wishlist ) {
			return $item_data;
		}

		$item = $wishlist->get_item( $nmgr_data[ 'wishlist_item_id' ] );

		if ( !$item ) {
			return $item_data;
		}

		$title = sprintf(
			/* translators: %s: wishlist type title */
			nmgr()->is_pro ? __( 'You are buying this item for this %s', 'nm-gift-registry' ) : __( 'You are buying this item for this %s', 'nm-gift-registry-lite' ),
			esc_html( nmgr_get_type_title() )
		);
		$data = array(
			'key' => sprintf(
				/* translators: %s: wishlist type title */
				nmgr()->is_pro ? __( 'For %s', 'nm-gift-registry' ) : __( 'For %s', 'nm-gift-registry-lite' ),
				esc_html( nmgr_get_type_title() )
			),
			'value' => nmgr_get_wishlist_link( $wishlist, array( 'title' => $title ) ),
			'display' => '',
		);
		$item_data[] = apply_filters( 'nmgr_get_item_data_content', $data, $item, $wishlist );

		return $item_data;
	}

	/**
	 * Displays simple notice on checkout page shipping section concerning the wishlist's shipping details
	 */
	public static function notify_wishlist_shipping() {
		$cart_wishlist_ids = nmgr()->is_pro ?
			is_nmgr_cart_shipping_to_wishlist_address( false ) :
			nmgr_get_wishlist_in_cart();

		if ( !empty( $cart_wishlist_ids ) ) {
			$default_msg = '';

			if ( nmgr()->is_pro ) {
				/* translators: %1$s: wishlist type title, %2$s: wishlist type title */
				$default_msg = sprintf( __( 'The items in your cart are for a %1$s and they would be shipped separately to the address of the %2$s owner.', 'nm-gift-registry' ),
					esc_html( nmgr_get_type_title() ),
					esc_html( nmgr_get_type_title() )
				);
			}

			$msg = apply_filters( 'nmgr_checkout_shipping_message', $default_msg );

			if ( $msg ) {
				wc_print_notice( $msg, 'notice' );
			}

			if ( nmgr()->is_pro ) {
				foreach ( $cart_wishlist_ids as $id ) {
					$wishlist = nmgr_get_wishlist( $id, true );

					if ( $wishlist ) {
						echo '<p>' . nmgr_get_wishlist_link( $wishlist ) . '<br>' .
						wc()->countries->get_formatted_address( $wishlist->get_shipping() ) . '</p>';
					}
				}
			}
		}
	}

	/**
	 * Add a wishlist's data as order itemmeta
	 */
	public static function add_wishlist_data_as_order_item_meta_data( $item, $cart_item_key, $values ) {
		$nmgr_data = nmgr_get_cart_item_data( $values, 'wishlist_item' );
		if ( !empty( $nmgr_data[ 'wishlist_item_id' ] ) ) {
			$wishlist_item = nmgr_get_wishlist_item( $nmgr_data[ 'wishlist_item_id' ] );
			if ( $wishlist_item ) {
				$wishlist_item->add_order_item_meta( $item );
				$item->save();
			}
		}
	}

	/**
	 * Add data concerning the wishlists in the order as order meta data
	 * once after the order has been saved
	 *
	 * This information would serve as the main data store from which we can
	 * perform gift registry related wishlist actions on the order later
	 *
	 * @param WC_Order $order The order object
	 */
	public static function add_wishlist_data_as_order_meta_data( $order ) {
		nmgr_add_order_meta_data( $order );
	}

	/**
	 * When a refund has been deleted, process payments for the wishlist items in the order
	 */
	public static function process_wishlist_item_payment_for_deleted_refunds( $refund_id, $order_id ) {
		self::process_wishlist_item_payment( $order_id );
	}

	/**
	 * Process payments for the wishlist items in an order
	 *
	 * This is the main entry point for actions relating to nm gift registry for WooCommerce orders.
	 *
	 * This function determines if we have any wishlist items in the order, if so it gets their data in the order
	 * and sets up some actions, based on whether the order is paid or refunded,
	 * that we can use to simplify the subsequent processing of these items in the order
	 *
	 * @param int $order_id The order id
	 */
	public static function process_wishlist_item_payment( $order_id ) {
		$order_object = new NMGR_Order_Data( $order_id );
		$order = $order_object->get_order();

		if ( !$order ) {
			return;
		}

		$order_wishlist_data = $order_object->get_meta();

		if ( !$order_wishlist_data && false === apply_filters( 'nmgr_do_order_payment_actions', false, $order ) ) {
			return;
		}

		/**
		 * At this point we confirm that there are wishlist items in the order
		 * so we can set up our actions
		 */
		$payment_cancelled_statuses = nmgr_get_payment_cancelled_order_statuses();
		$process_wishlist_item_payment_status = apply_filters(
			'nmgr_do_order_payment_complete',
			($order->get_date_paid() &&
			($order->is_paid() ||
			$order->has_status( $payment_cancelled_statuses ) ||
			doing_action( 'woocommerce_order_refunded' ) ||
			doing_action( 'woocommerce_refund_deleted' )
			) ),
			$order
		);

		if ( $process_wishlist_item_payment_status ) {
			do_action( 'nmgr_order_payment_complete', $order_id, $order_wishlist_data, $order );
		} else {
			do_action( 'nmgr_order_payment_incomplete', $order_id, $order_wishlist_data, $order );
		}
	}

	/**
	 * Update the purchased quantity of a wishlist item based on order status or action
	 *
	 * This function only runs if the 'purchased quantity' column is visible on the items table
	 * and if the order is paid for or refunded, that is typically on these statuses:
	 * - processing, completed, refunded, cancelled.
	 *
	 * Any other statuses of the order would not trigger an update of the wishlist item's purchased quantity.
	 * This is because the purchased quantity of a wishlist item is meant to be updated, as in the real shopping sense,
	 * only when an order has been paid for or refunded.
	 */
	public static function update_wishlist_item_purchased_quantity( $order_id ) {
		if ( !nmgr_get_option( 'display_item_purchased_quantity' ) ) {
			return;
		}

		$order_data = new NMGR_Order_Data( $order_id );
		$order = $order_data->get_order();
		$refunded_items = array();

		$order_meta = $order_data->get_meta();
		$stored_wishlist_ids = array_keys( $order_meta );

		foreach ( $stored_wishlist_ids as $wishlist_id ) {
			$wishlist = nmgr_get_wishlist( $wishlist_id );
			if ( $wishlist ) {
				foreach ( $wishlist->get_items() as $wishlist_item_object ) {
					/**
					 * If we don't have the order item id, we assume the item has been removed from the order
					 * (This might be due to a refund or something).
					 * So we simply delete the quantity reference of this item for the order if it exists
					 * and update its purchased quantity
					 */
					if ( !$order_data->get_order_item_id_for_wishlist_item_id( $wishlist_item_object->get_id() ) ) {
						$qr_meta = $wishlist_item_object->get_quantity_reference();
						$quantity_references = $qr_meta ? array_filter( $qr_meta ) : array();

						if ( isset( $quantity_references[ $order_id ] ) ) {
							unset( $quantity_references[ $order_id ] );

							$original_purchased_qty = $wishlist_item_object->get_purchased_quantity();

							$wishlist_item_object->set_quantity_reference( $quantity_references );
							$new_purchased_qty = array_sum( wp_list_pluck( $quantity_references, 'purchased_quantity' ) );
							// Item purchased quantity should always be updated after changing the quantity reference
							$wishlist_item_object->set_purchased_quantity( $new_purchased_qty );
							// Item purchase log should always be updated after updating the purchased quantity
							$qty_diff = $new_purchased_qty - $original_purchased_qty;
							if ( $qty_diff ) {
								$wishlist_item_object->add_purchase_log( $qty_diff, 'order', [ 'order_id' => $order_id ] );
							}

							$wishlist_item_object->save();

							/**
							 * The item was previously in the order because it has a quantity reference
							 * for the order id, but since it no longer has it, let's assume that the refunded
							 * quantity of the item has changed. (That is, all of the item is refunded).
							 * So we add the item to the refunded_items array.
							 */
							if ( $new_purchased_qty < $original_purchased_qty ) {
								$refunded_items[ $wishlist_item_object->get_id() ] = $original_purchased_qty - $new_purchased_qty;
							}
						}
					}
				}
			}
		}

		foreach ( $order_data->get_wishlist_item_ids() as $wishlist_item_id ) {
			$wishlist_item_object = nmgr_get_wishlist_item( $wishlist_item_id );
			$qr_meta = $wishlist_item_object->get_quantity_reference();

			/**
			 * array_filter was added in 2.1.0 to fix bug present from previous version when there is an empty array.
			 * Should probably remove this function in a later version
			 */
			$quantity_references = $qr_meta ? array_filter( $qr_meta ) : array();

			// If the quantity reference doesn't exist, flag this as a new order item
			$item_is_new = isset( $quantity_references[ $order_id ] ) ? false : true;

			// Get the order item object
			$order_item_id = $order_data->get_order_item_id_for_wishlist_item_id( $wishlist_item_id );
			$order_item_object = $order->get_item( $order_item_id );

			// Set a default quantity reference for new items
			$default_qr = array(
				'ordered_quantity' => 0,
				'refunded_quantity' => 0,
				'purchased_quantity' => 0,
			);

			/**
			 * If this item's quantity reference has not be set for this order, set the new one
			 * else get the one set for this order
			 */
			$qr = $item_is_new ? $default_qr : $quantity_references[ $order_id ];

			// Set a new quantity reference for the item for this order based on current item quantity properties
			$ordered_qty = absint( $order_item_object->get_quantity() );
			$refunded_qty = absint( $order->get_qty_refunded_for_item( $order_item_id ) );
			$new_qr = array(
				'ordered_quantity' => $ordered_qty,
				'refunded_quantity' => $refunded_qty,
				'purchased_quantity' => absint( $ordered_qty - $refunded_qty ),
			);

			// If the order payment is cancelled, update the quantity reference and
			// purchased quantity for the wishlist item
			if ( $order->has_status( nmgr_get_payment_cancelled_order_statuses() ) ) {
				$new_qr[ 'purchased_quantity' ] = 0;
			}

			/**
			 * If the item is not new and the new purchased quantity is less than the old purchased quantity,
			 * we assume the item is refunded  (or the order payment is cancelled)
			 * so we add it to the refunded_items array
			 */
			if ( !$item_is_new && ( $new_qr[ 'purchased_quantity' ] < $qr[ 'purchased_quantity' ] ) ) {
				$refunded_items[ $wishlist_item_object->get_id() ] = $qr[ 'purchased_quantity' ] - $new_qr[ 'purchased_quantity' ];
			}

			/**
			 * if the stored quantity reference is not equal to the new quantity reference
			 * the item purchased quantity might have changed, so update it
			 */
			if ( $qr !== $new_qr ) {
				$quantity_references[ $order_id ] = $new_qr;

				$wishlist_item_object->set_quantity_reference( $quantity_references );

				$original_purchased_qty = $wishlist_item_object->get_purchased_quantity();
				$new_purchased_qty = array_sum( wp_list_pluck( $quantity_references, 'purchased_quantity' ) );

				// Item purchased quantity should always be updated after changing the quantity reference
				$wishlist_item_object->set_purchased_quantity( $new_purchased_qty );

				// Item purchase log should always be updated after updating the purchased quantity
				$qty_diff = $new_purchased_qty - $original_purchased_qty;
				if ( $qty_diff ) {
					$wishlist_item_object->add_purchase_log( $qty_diff, 'order', [ 'order_id' => $order_id ] );
				}

				$wishlist_item_object->save();
			}
		}

		/**
		 * If we have items in the refunded_items array, set up the refund action.
		 */
		if ( !empty( $refunded_items ) ) {
			do_action( 'nmgr_order_items_refunded', $refunded_items, $order_id, $order_data->get_meta(), $order );
		}
	}

	/**
	 * After order payment, check if we should set the wishlist as fulfilled
	 *
	 * @param WC_Order $order_id
	 * @param array $order_wishlist_meta Wishlist data in order
	 */
	public static function maybe_set_wishlist_as_fulfilled( $order_id ) {
		$order_data = new NMGR_Order_Data( $order_id );
		// Loop through each wishlist in the order
		foreach ( $order_data->get_wishlist_ids() as $wishlist_id ) {
			$wishlist = nmgr_get_wishlist( $wishlist_id, true );

			/**
			 * If the wishlist is fulfilled we set a fulfilled date for the wishlist if it isn't already set
			 */
			if ( $wishlist->is_fulfilled() ) {
				if ( !$wishlist->get_date_fulfilled() ) {
					$wishlist->set_date_fulfilled( time() );
					$wishlist->save();
					/**
					 * Functions hooked into this action should typically only be run once (except in the case of refunds),
					 * as it is set when the wishlist has been detected as fulfilled based on the current order items
					 */
					do_action( 'nmgr_fulfilled_wishlist', $wishlist_id, $wishlist );
				}
			} else {
				/**
				 * If the wishlist is not fulfilled (perhaps because of refunds)
				 * but it already has a fulfilled date set, remove the date
				 */
				if ( $wishlist->get_date_fulfilled() ) {
					$wishlist->set_date_fulfilled( null );
					$wishlist->save();
				}
			}
		}
	}

	/**
	 * Make sure the wishlist data in the order matches the data in the wishlist itself
	 *
	 * This function is used on the following hooks:
	 * - nmgr_created_order: To update the ordered quantity for a wishlist
	 * item when an order is created
	 * - woocommerce_pre_payment_complete: To update the quantity reference when an order has
	 * been paid for. It is used on this hook as a precaution to make sure the update happens
	 * even though the hook 'woocommerce_payment_complete' also updates the quantity reference.
	 * -
	 */
	public static function do_process_wishlist_item_payment( $order_id ) {
		self::process_wishlist_item_payment( $order_id );
	}

	/**
	 * Remove wishlist items from the cart if their wishlist shipping address is
	 * required but not properly filled.
	 */
	public static function check_cart_items_for_shipping_address() {
		foreach ( wc()->cart->get_cart() as $cart_item_key => $cart_item ) {
			$nmgr_data = nmgr_get_cart_item_data( $cart_item, 'wishlist_item' );
			if ( $nmgr_data ) {
				$wishlist = nmgr_get_wishlist( $nmgr_data[ 'wishlist_id' ] );
				if ( $wishlist && $wishlist->needs_shipping_address() ) {
					wc()->cart->set_quantity( $cart_item_key, 0 );
					$message = sprintf(
						/* translators:
						 * 1: wishlist type title,
						 * 2: cart item name,
						 * 3: wishlist type title,
						 * 4: wishlist title
						 */
						nmgr()->is_pro ? __( 'The %1$s item %2$s has been removed from your cart as the shipping addresss for the %3$s %4$s is needed.', 'nm-gift-registry' ) : __( 'The %1$s item %2$s has been removed from your cart as the shipping addresss for the %3$s %4$s is needed.', 'nm-gift-registry-lite' ),
						esc_html( nmgr_get_type_title() ),
						'<strong>' . wp_kses_post( apply_filters( 'woocommerce_cart_item_name', $cart_item[ 'data' ]->get_name(), $cart_item, $cart_item_key ) ) . '</strong>',
						esc_html( nmgr_get_type_title() ),
						'<strong>' . esc_html( $wishlist->get_title() ) . '</strong>'
					);
					wc_add_notice( $message, 'error' );
				}
			}
		}
	}

	/**
	 * Show a button in the edit order screen to allow syncing of order wishlist data
	 * if the order has wishlist items.
	 *
	 * This function provides a last attempt to sync order wishlist data with
	 * stored wishlist data in situation where the standard methods have failed.
	 *
	 * The function has to be enabled manually by hooking into wordpress. E.g.
	 * add_action( 'woocommerce_order_actions_end', [ __CLASS__, 'sync_order_wishlists_metabox' ] );
	 *
	 * If enabled, the other method of this class 'manually_update_order_wishlist_data()'
	 * should also be enabled
	 *
	 */
	public static function sync_order_wishlists_metabox( $order_id ) {
		if ( nmgr_order_has_wishlist_items( $order_id ) ) {
			if ( nmgr()->is_pro ) {
				$text = __( 'You have wishlist items in this order. Click this button to manually update these wishlists to reflect their status in this order. It is only necessary to do this if you notice that the wishlist information in this order does not match the data in the wishlist itself.', 'nm-gift-registry' );
			} else {
				$text = __( 'You have wishlist items in this order. Click this button to manually update these wishlists to reflect their status in this order. It is only necessary to do this if you notice that the wishlist information in this order does not match the data in the wishlist itself.', 'nm-gift-registry-lite' );
			}

			echo '<li><button type="submit" name="nmgr_sync_order" value="' . esc_attr( $order_id ) . '" class="button button-primary nmgr-tip" title="' . esc_attr( $text ) . '">' . esc_html( nmgr()->is_pro ?
					__( 'Sync wishlist data', 'nm-gift-registry' ) :
					__( 'Sync wishlist data', 'nm-gift-registry-lite' )  ) . '</button></li>';
		}
	}

	/**
	 * Sync order wishlist data with stored wishlist data
	 *
	 * This function checks if the order id is in the $_POST variable 'nmgr_sync_order' and
	 * performs the sync if so.
	 *
	 * It has to be manually enabled by hooking into wordpress. E.g.
	 * add_action( 'wp_loaded', [ __CLASS__, 'manually_update_order_wishlist_data' ] );
	 *
	 * It is typically enabled with the other method of this class 'sync_order_wishlists_metabox()'
	 */
	public static function manually_update_order_wishlist_data() {
		if ( !empty( $_POST[ 'nmgr_sync_order' ] ) ) {
			nmgr_sync_order_data( ( int ) $_POST[ 'nmgr_sync_order' ] );
		}
	}

	/**
	 * Show the wishlist title as item meta data whenever the item meta data is displayed
	 */
	public static function show_order_item_meta_data( $meta_array, $item ) {
		$nmgr = $item->get_meta( 'nm_gift_registry' );
		if ( $nmgr && in_array( $item->get_type(), [ 'line_item', 'shipping' ] ) ) {
			foreach ( $meta_array as $id => $meta ) {
				if ( 'nmgr_wishlist_title' === $meta->key ) {
					$wishlist = nmgr_get_wishlist( $nmgr[ 'wishlist_id' ], true );

					if ( $wishlist ) {
						$title = sprintf(
							/* translators: %s: wishlist type title */
							nmgr()->is_pro ? __( 'This item is bought for this %s', 'nm-gift-registry' ) : __( 'This item is bought for this %s', 'nm-gift-registry-lite' ),
							nmgr_get_type_title()
						);

						$show_link = true;
						if ( is_admin() ) {
							$screen = function_exists( 'get_current_screen' ) ? get_current_screen() : null;
							if ( !$screen || ($screen && !in_array( $screen->id, [ 'shop_order', 'edit-shop_order' ] )) ) {
								$show_link = false;
							}
						}

						$meta_array[ $id ]->display_key = nmgr_get_type_title( 'c' );
						$meta_array[ $id ]->display_value = $show_link ?
							nmgr_get_wishlist_link( $wishlist, [ 'title' => $title ] ) :
							$wishlist->get_title();
					}
				}

				// pro version
				if ( 'nmgr_shipping_address' === $meta->key ) {
					$meta_array[ $id ]->display_key = __( 'Shipping address', 'nm-gift-registry' );
				}
			}
		}
		return $meta_array;
	}

	public static function add_wishlist_product_to_cart_from_product_page() {
		$item_id = filter_input( INPUT_GET, 'nmgr_item_id' );

		if ( !$item_id ) {
			return;
		}

		$item = nmgr_get_wishlist_item( $item_id );
		$product = $item ? $item->get_product() : false;

		if ( !$product ) {
			return;
		}

		$wishlist = $item->get_wishlist();
		$view = nmgr()->items_view( $wishlist );
		$view->set_item( $item );
		$button = $view->get_item_add_to_cart_button();

		$message = sprintf(
			/* translators: 1: product name, 2: wishlist type title, 3: wishlist title */
			nmgr()->is_pro ? __( 'Purchase %1$s for the %2$s %3$s.', 'nm-gift-registry' ) : __( 'Purchase %1$s for the %2$s %3$s.', 'nm-gift-registry-lite' ),
			'<strong>' . $product->get_name() . '</strong>',
			nmgr_get_type_title(),
			'<strong>' . $wishlist->get_title() . '</strong>'
		);
		?>
		<style>
			.nmgr-atc-product-page {
				background-color: #f8f8f8;
				display: flex;
				align-items: center;
				justify-content: space-between;
				margin: 20px 0;
				padding: 20px;
			}
		</style>
		<div class="nmgr-atc-product-page">
			<div class="nmgr-atc-product-page-msg">
				<?php echo wp_kses_post( $message ); ?>
			</div>
			<div class="nmgr-atc-product-page-btn">
				<?php echo $button; ?>
			</div>
		</div>
		<?php
	}

	public static function show_custom_order_notice() {
		global $post, $pagenow, $typenow;

		if ( 'post.php' === $pagenow && 'shop_order' === $typenow ) {
			$order = wc_get_order( $post );
			if ( $order && $order->is_created_via( 'nmgr_wishlist' ) ) {
				echo '<div class="notice-info notice"><p>' . wp_kses_post( nmgr_get_custom_order_notice() ) . '</p></div>';
			}
		}
	}

	public static function show_custom_order_notice_in_email( $order, $sent_to_admin, $plain_text ) {
		if ( $order && $order->is_created_via( 'nmgr_wishlist' ) ) {
			$notice = nmgr_get_custom_order_notice();
			echo $plain_text ? (esc_html( $notice ) . "\n\n") : ('<p>' . esc_html( $notice ) . '</p>');
		}
	}

}
