<?php

/**
 * Sync
 */
use NmeriMedia\V2\Scripts;

defined( 'ABSPATH' ) || exit;

class NMGR_Scripts {

	/**
	 * Suffix to append to file name before file extension e.g. 'min' for minified
	 *
	 * @var string
	 */
	private static $suffix;

	/**
	 * Localized script handles
	 *
	 * @var array
	 */
	private static $inline_scripts = array();

	public static function run() {
		self::$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		// 999 priority ensures we hook our scripts after woocommerce's and any other conflicting plugin scripts
		add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_scripts' ), 999 );
		add_action( 'wp_enqueue_scripts', array( __CLASS__, 'frontend_scripts' ), 999 );

		add_action( 'wp_print_scripts', array( __CLASS__, 'add_inline_scripts' ), 5 );
		add_action( 'wp_print_footer_scripts', array( __CLASS__, 'add_inline_scripts' ), 5 );

		add_action( 'wp_footer', 'nmgr_include_sprite_file' );
		add_action( 'admin_footer', 'nmgr_include_sprite_file' );
	}

	public static function frontend_scripts() {
		self::register_common_scripts();
		self::register_frontend_scripts();

		$scripts = new Scripts();
		$scripts->enqueue_style( 'bootstrap' );
		$scripts->enqueue_script( 'popper' );
		$scripts->enqueue_script( 'bootstrap' );

		wp_enqueue_style( 'nmgr-frontend' );
		wp_enqueue_script( 'nmgr-frontend' );

		if ( nmgr()->is_pro ) {
			wp_add_inline_style( 'nmgr-frontend', self::get_frontend_inline_style() );
		}
	}

	public static function admin_scripts() {
		global $typenow, $pagenow;

		self::register_common_scripts();
		self::register_admin_scripts();

		if ( is_nmgr_admin() || ('product' === $typenow && 'edit.php' === $pagenow) ) {
			wp_enqueue_style( 'nmgr-admin' );
			wp_enqueue_script( 'nmgr-admin' );
		}
	}

	/**
	 * Register scripts used on both admin and frontend
	 *
	 * Because woocommerce also registers some of these scripts, we have to check
	 * if the script it registered so that we can use woocommerce's own and not override it
	 * with our own or add our script to the page if it already exists
	 */
	private static function register_common_scripts() {
		// Styles
		wp_register_style( 'nmgr-jquery-tiptip', nmgr()->url . 'assets/css/vendor/jquery-tiptip' . self::$suffix . '.css', array() );
		wp_register_style( 'nmgr-datatables', nmgr()->url . 'assets/css/vendor/datatables' . self::$suffix . '.css', array(), nmgr()->version );

		if ( !wp_style_is( 'select2', 'registered' ) ) {
			wp_register_style( 'select2', nmgr()->url . 'assets/css/vendor/select2.css', array() );
		}

		// Scripts
		if ( !wp_script_is( 'jquery-blockui', 'registered' ) ) {
			wp_register_script( 'jquery-blockui', nmgr()->url . 'assets/js/vendor/jquery.blockUI' . self::$suffix . '.js', array( 'jquery' ), '2.70', true );
		}

		if ( !wp_script_is( 'jquery-tiptip', 'registered' ) ) {
			wp_register_script( 'jquery-tiptip', nmgr()->url . 'assets/js/vendor/jquery.tipTip' . self::$suffix . '.js', array( 'jquery' ), nmgr()->version, true );
		}

		// We're forced to use selectWoo instead of select2 because woocommerce uses it for shipping fields
		if ( !wp_script_is( 'selectWoo', 'registered' ) ) {
			wp_register_script( 'selectWoo', nmgr()->url . 'assets/js/vendor/selectWoo.full' . self::$suffix . '.js', array( 'jquery' ), '1.0.6', true );
		}

		wp_register_script( 'nmgr-datatables', nmgr()->url . 'assets/js/vendor/datatables' . self::$suffix . '.js', array( 'jquery' ), nmgr()->version, true );
	}

	private static function register_admin_scripts() {
		$version = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? date( 'H:i:s' ) : nmgr()->version;

		wp_register_style( 'nmgr-admin', nmgr()->url . 'assets/css/admin' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip', 'nmgr-datatables' ), $version );
		wp_register_script( 'nmgr-admin', nmgr()->url . 'assets/js/admin' . self::$suffix . '.js', array( 'jquery', 'selectWoo', 'nmgr-datatables', 'jquery-blockui', 'jquery-ui-datepicker', 'jquery-tiptip' ), $version, true );
	}

	private static function register_frontend_scripts() {
		$version = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? date( 'H:i:s' ) : nmgr()->version;

		wp_register_style( 'nmgr-frontend', nmgr()->url . 'assets/css/frontend' . self::$suffix . '.css', array( 'nmgr-jquery-tiptip', 'select2', 'nmgr-datatables' ), $version );
		wp_register_script( 'nmgr-frontend', nmgr()->url . 'assets/js/frontend' . self::$suffix . '.js', array( 'jquery', 'jquery-tiptip', 'wc-add-to-cart-variation', 'nmgr-datatables', 'jquery-blockui', 'selectWoo', 'wc-country-select', 'wc-address-i18n', 'jquery-ui-datepicker' ), $version, true );
	}

	private static function get_script_data( $handle = '' ) {
		$data = array();
		$nmgr_global = isset( $GLOBALS[ 'nmgr' ] ) ? $GLOBALS[ 'nmgr' ] : '';
		$ajax_url = admin_url( 'admin-ajax.php' );

		// Parameters that can be used by various scripts
		$global_params = array(
			'global' => $nmgr_global,
			'is_pro' => nmgr()->is_pro,
			'ajax_url' => $ajax_url,
			'nonce' => wp_create_nonce( 'nmgr' ), // Generic nonce for the application,
			'datepicker_options' => apply_filters( 'nmgr_datepicker_options', [
				'altFormat' => 'yy-mm-dd',
				'changeMonth' => true,
				'changeYear' => true,
				'styleDatepicker' => apply_filters_deprecated(
					'nmgr_style_datepicker',
					[ true ],
					'2.4.3',
					'nmgr_datepicker_options'
				),
			] ),
			'responsive_tables' => apply_filters( 'nmgr_responsive_tables', array(
				'items_table' => array( 'selector' => '.nmgr-items-table', 'events' => array( 'nmgr_tab_shown', 'nmgr_items_reloaded' ) ),
				'orders_table' => array( 'selector' => '.nmgr-orders-table', 'events' => array( 'nmgr_tab_shown', 'nmgr_orders_reloaded' ) ),
				'wishlists_table' => array( 'selector' => '.nmgr-wishlists-table' ),
			) ),
			'datatables_tables' => nmgr_get_datatables(),
		);

		$data[ 'nmgr-frontend' ] = array(
			'global' => $global_params,
			'nonce' => wp_create_nonce( 'nmgr-frontend' ),
			'images' => array(
				'i18n_invalid_type_text' => nmgr()->is_pro ?
				esc_attr__( 'The uploaded file is not a valid image. Please try again.', 'nm-gift-registry' ) :
				'',
			),
		);


		$data[ 'nmgr-admin' ] = array(
			'global' => $global_params,
			'search_users_nonce' => wp_create_nonce( 'nmgr-search-users' ),
			'search_products_nonce' => wp_create_nonce( 'nmgr-search-products' ),
			'i18n_select_state_text' => nmgr()->is_pro ?
			esc_attr__( 'Select an option...', 'nm-gift-registry' ) :
			esc_attr__( 'Select an option...', 'nm-gift-registry-lite' ),
			'i18n_required_text' => nmgr()->is_pro ?
			esc_attr__( 'required', 'nm-gift-registry' ) :
			esc_attr__( 'required', 'nm-gift-registry-lite' ),
		);

		if ( is_a( wc()->countries, 'WC_Countries' ) ) {
			$countries_params = [
				'countries' => wp_json_encode( array_merge(
						WC()->countries->get_allowed_country_states(),
						WC()->countries->get_shipping_country_states()
				) ),
				'locale' => wp_json_encode( WC()->countries->get_country_locale() ),
				'locale_fields' => wp_json_encode( WC()->countries->get_country_locale_field_selectors() ),
			];

			$data[ 'nmgr-admin' ] = array_merge( $data[ 'nmgr-admin' ], $countries_params );
		}

		$filtered_data = apply_filters( 'nmgr_script_data', $data );

		if ( $handle ) {
			return isset( $filtered_data[ $handle ] ) ? $filtered_data[ $handle ] : false;
		}

		return $filtered_data;
	}

	public static function add_inline_scripts() {
		$handles = array_keys( self::get_script_data() );
		$global_inline_script_added = false;

		foreach ( $handles as $handle ) {
			/**
			 * We have to use this condition because this function is hooked to both wp_print_scripts and
			 * wp_print_footer_scripts so it runs twice and we dont want to add the inline scripts twice so we
			 * make sure that once it is added, it is not added again.
			 */
			if ( !in_array( $handle, self::$inline_scripts, true ) && wp_script_is( $handle, 'enqueued' ) ) {
				$data = self::get_script_data( $handle );

				if ( isset( $data[ 'global' ] ) ) {
					if ( false === $global_inline_script_added ) {
						wp_add_inline_script( $handle, 'var nmgr_global_params = ' . json_encode( $data[ 'global' ] ), 'before' );
						$global_inline_script_added = true;
					}

					if ( $global_inline_script_added ) {
						unset( $data[ 'global' ] );
					}
				}

				if ( !empty( $data ) ) {
					$name = str_replace( '-', '_', $handle ) . '_params';
					wp_add_inline_script( $handle, 'var ' . $name . ' = ' . json_encode( $data ), 'before' );
				}

				self::$inline_scripts[] = $handle;
			}
		}
	}

	/**
	 * Inline styles used wtih frontend styles
	 *
	 * @return string
	 */
	public static function get_frontend_inline_style() {
		$styles = array();
		$post_thumbnail_size = nmgr()->post_thumbnail_size() / 16; // convert px to em

		$styles[ 'images' ] = "
			#nmgr-images .nmgr-thumbnail {
				max-width: {$post_thumbnail_size}em;
				max-height: {$post_thumbnail_size}em;
			}

			@media screen and (min-width: 768px) {
				#nmgr-images.show-bg .featured-image-wrapper {
					height: calc({$post_thumbnail_size}em * 60/100);
				}
				#nmgr-images .featured-image-wrapper .nmgr-thumbnail {
					width: {$post_thumbnail_size}em;
					height: {$post_thumbnail_size}em;
				}
			}
			";

		return apply_filters( 'nmgr_frontend_inline_style', implode( ' ', $styles ) );
	}

}
