<?php
/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

class NMGR_Templates {

	public static function run() {
		// Setup templates with woocommerce and wordpress
		add_filter( 'is_woocommerce', array( __CLASS__, 'is_woocommerce' ) );
		add_filter( 'body_class', array( __CLASS__, 'body_class' ) );
		add_filter( 'template_include', array( __CLASS__, 'include_single_template' ) );
		add_filter( 'template_include', array( __CLASS__, 'include_archive_template' ) );
		add_action( 'the_post', 'nmgr_set_global_var', -1 );
		add_action( 'add_meta_boxes', 'nmgr_set_global_var', -1 );
		add_action( 'template_redirect', 'nmgr_set_global_var', -1 );
		add_action( 'woocommerce_before_template_part', array( __CLASS__, 'set_wc_template_global_variable' ), -1, 4 );
		add_action( 'woocommerce_after_template_part', array( __CLASS__, 'unset_wc_template_global_variable' ), -1 );

		// Single wishlist page content
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_title' ), 20 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_display_name' ), 30 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_event_date' ), 40 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_description' ), 50 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_notices' ), 60 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_items' ), 70 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_share_links' ), 80 );
		add_action( 'nmgr_wishlist', array( __CLASS__, 'single_show_copy_link' ), 90 );

		// After wishlist items view
		add_action( 'nmgr_after_items', array( __CLASS__, 'after_items_show_items_total_cost' ), 10, 4 );
		add_action( 'nmgr_after_items', array( __CLASS__, 'after_items_show_items_actions' ), 20, 4 );
		add_action( 'nmgr_after_items_actions', array( __CLASS__, 'after_items_actions_show_add_items_button' ) );

		// Setup account page
		$page = nmgr_get_account_details( 'slug' );

		if ( $page ) {
			add_filter( 'woocommerce_get_query_vars', array( __CLASS__, 'add_account_query_var' ) );
			add_filter( 'woocommerce_account_menu_items', array( __CLASS__, 'add_account_menu_item' ), 10, 1 );
			add_filter( "woocommerce_endpoint_{$page}_title", array( __CLASS__, 'endpoint_title' ), 10, 2 );
			add_action( "woocommerce_account_{$page}_endpoint", array( __CLASS__, 'account_endpoint' ), 10 );
		}

		// Account page content
		add_action( 'woocommerce_account_dashboard', array( __CLASS__, 'show_wishlist_dashboard_text' ), 10 );

		// Modify Wishlist data properties
		add_filter( 'nmgr_get_prop', array( __CLASS__, 'get_wishlist_property' ), 10, 4 );

		// Sidebar
		add_action( 'nmgr_sidebar', array( __CLASS__, 'show_theme_sidebar' ), 10 );

		add_filter( 'nmgr_delete_item_notice', array( __CLASS__, 'notify_of_item_purchased_status' ), 10, 2 );

		// Archives
		add_action( 'nmgr_archive_header', array( __CLASS__, 'show_archive_header' ), 10, 2 );
		add_action( 'nmgr_after_archive_title', array( __CLASS__, 'show_archive_results_count_and_page_number' ), 10, 2 );
		add_action( 'nmgr_archive_no_results_found', array( __CLASS__, 'no_search_results_notification' ) );

		add_action( 'wp_footer', [ __CLASS__, 'set_toast' ] );
		add_action( 'admin_footer', [ __CLASS__, 'set_toast' ] );
		add_action( 'nmerimedia_component_init', [ __CLASS__, 'set_in_modal_global_variable' ], -1 );
		add_action( 'nmerimedia_component_output', [ __CLASS__, 'unset_in_modal_global_variable' ], -1 );
		add_filter( 'nmgr_requested_fields', [ __CLASS__, 'use_html5_date_input_for_event_date_in_modal' ] );
	}

	public static function single_show_copy_link( $wishlist ) {
		if ( 'publish' !== $wishlist->get_status() ) {
			return;
		}
		?>
		<div class="nmgr-copy-wrapper nmgr-tip"
				 title="<?php
				 printf(
					 /* translators: %s: wishlist type title */
					 nmgr()->is_pro ? esc_attr__( 'Copy your %s url', 'nm-gift-registry' ) : esc_attr__( 'Copy your %s url', 'nm-gift-registry-lite' ),
					 esc_html( nmgr_get_type_title( '', 0, $wishlist->get_type() ) )
				 );
				 ?>">
			<svg width="1.2em" height="1.2em" clip-rule="evenodd" fill-rule="evenodd" stroke-linejoin="round" stroke-miterlimit="2" viewBox="0 0 24 24" xmlns="http://www.w3.org/2000/svg"><path d="m6 18h-3c-.48 0-1-.379-1-1v-14c0-.481.38-1 1-1h14c.621 0 1 .522 1 1v3h3c.621 0 1 .522 1 1v14c0 .621-.522 1-1 1h-14c-.48 0-1-.379-1-1zm1.5-10.5v13h13v-13zm9-1.5v-2.5h-13v13h2.5v-9.5c0-.481.38-1 1-1z" fill-rule="nonzero"/></svg>
			<div class="nmgr-link nmgr-copy"><?php echo esc_html( $wishlist->get_permalink() ); ?> </div>
		</div>
		<?php
	}

	public static function set_in_modal_global_variable( $comp ) {
		if ( 'modal' === $comp->get_type() ) {
			nmgr_get_global_var()->is_modal = true;
		}
	}

	public static function unset_in_modal_global_variable( $comp ) {
		if ( 'modal' === $comp->get_type() ) {
			nmgr_get_global_var()->is_modal = false;
		}
	}

	public static function use_html5_date_input_for_event_date_in_modal( $fields ) {
		if ( isset( $fields[ 'event_date' ] ) && is_nmgr_modal() ) {
			$fields[ 'event_date' ][ 'type' ] = 'date';
			$fields[ 'event_date' ][ 'custom_attributes' ][ 'pattern' ] = '[0-9]{4}-(0[1-9]|1[012])-(0[1-9]|1[0-9]|2[0-9]|3[01])';
			$fields[ 'event_date' ][ 'placeholder' ] = nmgr()->is_pro ?
				__( 'yyyy-mm-dd', 'nm-gift-registry' ) :
				__( 'yyyy-mm-dd', 'nm-gift-registry-lite' );

			if ( isset( $fields[ 'event_date_display' ] ) ) {
				unset( $fields[ 'event_date_display' ] );
			}
		}
		return $fields;
	}

	/**
	 * Declare certain NM Gift Registry pages as woocommerce pages
	 *
	 * @param bool $boolean
	 * @return boolean
	 */
	public static function is_woocommerce( $boolean ) {
		return is_nmgr_wishlist() || $boolean;
	}

	/**
	 * Handles loading templates for viewing a single wishlist
	 *
	 * NM Gift Registry is a custom post type so a single wishlist can be viewed using the file
	 * single-nm_gift_registry.php in the theme root folder.
	 * This would be loaded if the file is not found in the theme's nm_gift_registry plugin folder.
	 * Finally the file will be loaded from the nm_gift_registry plugin's templates folder if it is
	 * not found in the two locations in the theme.
	 *
	 * @return string Template filepath
	 */
	public static function include_single_template( $template ) {
		if ( is_singular( 'nm_gift_registry' ) ) {
			$single_template_array = array(
				nmgr()->theme_path() . 'single-nm_gift_registry.php', // theme folder for plugin templates
				'single-nm_gift_registry.php', // theme root
			);

			$single_template = locate_template( $single_template_array );
			$template = $single_template ? $single_template : nmgr()->template_path() . 'single-nm_gift_registry.php';
		}

		return $template;
	}

	/**
	 * Handles loading templates for viewing wishlist archives
	 *
	 * NM Gift Registry is a custom post type so archive results can be viewed using the
	 * file archive-nm_gift_registry.php which is loaded first from the theme's nm_gift_registry
	 * plugin folder if found. Else it is loaded from the theme's root folder. If not found here,
	 * it is finally loaded from the plugin templates folder.
	 *
	 * @return string Template filepath
	 */
	public static function include_archive_template( $template ) {
		if ( is_nmgr_archive() && !nmgr_post_content_has_shortcode( '[nmgr_archive]' ) ) {
			$archive_template_array = array(
				nmgr()->theme_path() . 'archive-nm_gift_registry.php', // theme folder for plugin templates
				'archive-nm_gift_registry.php', // theme root
			);

			$archive_template = locate_template( $archive_template_array );
			$template = $archive_template ? $archive_template : nmgr()->template_path() . 'archive-nm_gift_registry.php';
		}

		return $template;
	}

	/**
	 * Add body classes to identify all nmgr pages
	 *
	 * Page specific class is not added to the single wishlist page  as a custom post type
	 * because wordpress already adds the class 'single-nm_gift_registry' to it
	 *
	 * Classes added here follow this format {page-type}nm_gift_registry
	 *
	 * @param type $classes
	 * @return string
	 */
	public static function body_class( $classes ) {
		if ( is_nmgr() ) {
			$classes[] = 'nm_gift_registry';
		}

		if ( is_nmgr_search() ) {
			$classes[] = 'search-nm_gift_registry';
		}

		if ( is_nmgr_wishlist() ) {
			$classes[] = 'nm_gift_registry-wishlist';
		}

		if ( is_nmgr_wishlist_page( 'archive' ) ) {
			$classes[] = 'nm_gift_registry-archive';
		}

		if ( is_nmgr_account_section() ) {
			$classes[] = 'nm_gift_registry';
			$classes[] = 'account-nm_gift_registry';
		}

		return $classes;
	}

	public static function set_global_variable() {
		_deprecated_function( __METHOD__, '3.0.0', 'nmgr_set_global_var()' );
		nmgr_set_global_var();
	}

	public static function set_wc_template_global_variable( $name, $path, $located, $args ) {
		$in_templates = isset( $GLOBALS[ 'nmgr_templates' ] ) ? $GLOBALS[ 'nmgr_templates' ] : array();

		$in_templates[ $name ] = array(
			'template_name' => $name,
			'template_path' => $path,
			'located' => $located,
			'args' => $args
		);

		$GLOBALS[ 'nmgr_templates' ] = $in_templates;
	}

	public static function unset_wc_template_global_variable( $template_name ) {
		if ( isset( $GLOBALS[ 'nmgr_templates' ], $GLOBALS[ 'nmgr_templates' ][ $template_name ] ) ) {
			unset( $GLOBALS[ 'nmgr_templates' ][ $template_name ] );
		}
	}

	/**
	 * Show the wishlist title on the single wishlist page
	 *
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function single_show_title( $wishlist ) {
		printf( '<h2 class="nmgr-title nmgr-text-center entry-title">%s</h2>', esc_html( $wishlist->get_title() ) );
	}

	/**
	 * Show the wishlist display name on the single wishlist page
	 *
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function single_show_display_name( $wishlist ) {
		if ( $wishlist->get_display_name() ) {
			printf( '<h3 class="nmgr-display-name nmgr-text-center">%s</h3>', esc_html( $wishlist->get_display_name() ) );
		}
	}

	/**
	 * Show the wishlist event date on the single wishlist page
	 *
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function single_show_event_date( $wishlist ) {
		if ( $wishlist->is_type( 'wishlist' ) ) {
			return;
		}

		$date = nmgr_format_date( $wishlist->get_event_date() );
		if ( $date ) :

			if ( nmgr_user_has_wishlist( $wishlist ) ) {
				$expiry_days = $wishlist->get_expiry_days();
				$abs_days = absint( $expiry_days );
				$days_notice = '';

				if ( $expiry_days > 0 ) {
					$days_notice = sprintf(
						/* translators: %d: wishlist event days */
						nmgr()->is_pro ? _n( '%d day to your event', '%d days to your event', $abs_days, 'nm-gift-registry' ) : _n( '%d day to your event', '%d days to your event', $abs_days, 'nm-gift-registry-lite' ),
						$abs_days
					);
					$expiry_days = "+$expiry_days";
				} elseif ( $expiry_days < 0 ) {
					$days_notice = sprintf(
						/* translators: %d: wishlist event days */
						nmgr()->is_pro ? _n( '%d day after your event', '%d days after your event', $abs_days, 'nm-gift-registry' ) : _n( '%d day after your event', '%d days after your event', $abs_days, 'nm-gift-registry-lite' ),
						$abs_days
					);
				} else {
					$days_notice = nmgr()->is_pro ?
						__( 'Your event is today', 'nm-gift-registry' ) :
						__( 'Your event is today', 'nm-gift-registry-lite' );
					$expiry_days = $days_notice;
				}
			}
			?>

			<p class="nmgr-event-date nmgr-text-center">
				<span class="nmgr-date-text">
					<?php
					echo esc_html( nmgr()->is_pro ?
							__( 'Event date', 'nm-gift-registry' ) :
							__( 'Event date', 'nm-gift-registry-lite' )
					);

					echo ': ' . esc_html( $date );
					?>
				</span>
				<?php if ( nmgr_user_has_wishlist( $wishlist ) ) : ?>
					<span class="nm badge nmgr-tip"
								style="background-color:#ccc; vertical-align: text-top; margin-left: 5px;"
								title="<?php esc_attr_e( $days_notice ); ?>"><?php echo esc_html( $expiry_days ); ?></span>
							<?php endif; ?>
			</p>

			<?php
		endif;
	}

	/**
	 * Show the wishlist description on the single wishlist page
	 *
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function single_show_description( $wishlist ) {
		if ( $wishlist->get_description() ) {
			printf( '<div class="nmgr-description nmgr-text-center">%s</div>', wp_kses_post( wpautop( $wishlist->get_description() ) ) );
		}
	}

	/**
	 * Show woocommerce notices if available
	 */
	public static function single_show_notices() {
		/**
		 * We first check if the functions exists to prevent fatal error for
		 * 'wc_print_notices()' not found when saving a page with the shortcode
		 * in the admin area
		 */
		if ( function_exists( 'woocommerce_output_all_notices' ) && function_exists( 'wc_print_notices' ) ) {
			woocommerce_output_all_notices();
		}
	}

	/**
	 * Show the wishlist items on the single wishlist page
	 *
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function single_show_items( $wishlist ) {
		if ( $wishlist->is_type( 'gift-registry' ) && $wishlist->is_fulfilled() &&
			nmgr_get_type_option( $wishlist->get_type(), 'hide_fulfilled_items' ) ) {
			return;
		}

		nmgr_get_items_template( $wishlist, true );
	}

	/**
	 * Show share links on the single wishlist page
	 */
	public static function single_show_share_links( $wishlist ) {
		nmgr_get_share_template( $wishlist, true );
	}

	/**
	 * Show total cost of all items
	 *
	 * @param array $items Wishlist items
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function after_items_show_items_total_cost( $items, $wishlist, $items_args, $view ) {
		if ( $wishlist->is_type( 'gift-registry' ) &&
			in_array( 'total_cost', array_keys( $view->get_parts_data() ) ) &&
			(is_nmgr_admin() || nmgr_user_has_wishlist( $wishlist )) ) {

			function nmgr_item_totals_notice( $title ) {
				return nmgr_get_svg( array(
					'icon' => 'info',
					'class' => 'align-with-text nmgr-tip',
					'style' => 'margin-left:4px;',
					'size' => '.8',
					'title' => $title,
					'fill' => '#999',
					) );
			}

			$rows = [
				'amount_purchased' => [
					'priority' => 10,
					'label' => (nmgr()->is_pro ?
					__( 'Amount purchased', 'nm-gift-registry' ) :
					__( 'Amount purchased', 'nm-gift-registry-lite' )) .
					nmgr_item_totals_notice( sprintf(
							/* translators: %s: wishlist type title */
							nmgr()->is_pro ? __( 'The value of all the purchased items in this %s.', 'nm-gift-registry' ) : __( 'The value of all the purchased items in this %s.', 'nm-gift-registry-lite' ),
							nmgr_get_type_title()
					) ) . ' :',
					'content' => wc_price( $wishlist->get_total_purchased_amount() ),
					'show' => $wishlist->has_items(),
				],
				'amount_needed' => [
					'priority' => 20,
					'label' => (nmgr()->is_pro ?
					__( 'Amount needed', 'nm-gift-registry' ) :
					__( 'Amount needed', 'nm-gift-registry-lite' )) .
					nmgr_item_totals_notice( sprintf(
							/* translators: %s: wishlist type title */
							nmgr()->is_pro ? __( 'The amount needed to completely purchase all the items in this %s.', 'nm-gift-registry' ) : __( 'The amount needed to completely purchase all the items in this %s.', 'nm-gift-registry-lite' ),
							nmgr_get_type_title()
					) ) . ' :',
					'content' => wc_price( $wishlist->get_total_unpurchased_amount() ),
					'show' => $wishlist->has_items(),
					'class' => [ 'nmgr-grey' ],
				],
				'total' => [
					'priority' => 30,
					'label' => (nmgr()->is_pro ?
					__( 'Total', 'nm-gift-registry' ) :
					__( 'Total', 'nm-gift-registry-lite' )) .
					nmgr_item_totals_notice( sprintf(
							/* translators: %s: wishlist type title */
							nmgr()->is_pro ? __( 'The total cost of all the items in this %s.', 'nm-gift-registry' ) : __( 'The total cost of all the items in this %s.', 'nm-gift-registry-lite' ),
							nmgr_get_type_title()
					) ) . ' :',
					'content' => wc_price( $wishlist->get_total() )
				],
			];

			$filtered_rows = apply_filters( 'nmgr_items_total_table_rows', $rows, $view );
			$showing_rows = nmgr_get_elements_to_show( $filtered_rows );
			uasort( $showing_rows, 'nmgr_sort_by_priority' );

			nmgr_template( 'account/items/items-total-cost.php',
				array(
					'wishlist' => $wishlist,
					'rows' => $showing_rows
				)
			);
		}
	}

	/**
	 * Show (add/save) actions relating to all items
	 *
	 * @param array $items Wishlist items
	 * @param NMGR_Wishlist $wishlist
	 */
	public static function after_items_show_items_actions( $items, $wishlist, $items_args, $view ) {
		nmgr_template( 'account/items/items-actions.php',
			array(
				'items' => $items,
				'wishlist' => $wishlist,
				'items_args' => $items_args,
				'view' => $view,
		) );
	}

	/**
	 * Show button for adding an item to the wishlist
	 *
	 * On the frontend, this button simply redirects to the shop page
	 */
	public static function after_items_actions_show_add_items_button() {
		if ( is_nmgr_admin() ) {
			?>
			<button type="button"
							class="button nmgr-add-items-action" data-bs-toggle="modal" data-bs-target="#nmgr-add-items-dialog">
								<?php
								echo esc_html( nmgr()->is_pro ?
										__( 'Add item(s)', 'nm-gift-registry' ) :
										__( 'Add item(s)', 'nm-gift-registry-lite' )
								);
								?>
			</button>
			<?php
		}
	}

	/**
	 * Show link to wishlist endpoint url on woocommerce account dashboard
	 */
	public static function show_wishlist_dashboard_text() {
		foreach ( [ 'gift-registry', 'wishlist' ] as $type ) {
			if ( is_nmgr_enabled( $type ) ) {
				printf(
					/* translators: 1: wishlist module account url, 2: wishlist type title */
					nmgr()->is_pro ? wp_kses_post( __( '<p>You can also manage your <a href="%1$s">%2$s</a>.</p>', 'nm-gift-registry' ) ) : wp_kses_post( __( '<p>You can also manage your <a href="%1$s">%2$s</a>.</p>', 'nm-gift-registry-lite' ) ),
					esc_url( nmgr_get_url( $type, 'home' ) ),
					esc_html( nmgr_get_type_title( '', '', $type ) )
				);
			}
		}
	}

	/**
	 *
	 * Add NM Gift Registry account query variable to woocommerce query variables
	 *
	 * @param array $query_vars Query vars
	 * @return array
	 */
	public static function add_account_query_var( $query_vars ) {
		$account_slug = nmgr_get_account_details( 'slug' );
		if ( $account_slug ) {
			$query_vars[ $account_slug ] = $account_slug;
		}
		return $query_vars;
	}

	/**
	 * Set up NM Gift Registry menu item name, slug and position in woocommerce my account nav menu
	 */
	public static function add_account_menu_item( $items ) {
		if ( !is_nmgr_enabled() ) {
			return $items;
		}

		$account_slug = nmgr_get_account_details( 'slug' );

		if ( $account_slug && isset( $items[ 'customer-logout' ] ) ) {
			$index = -1;

			foreach ( $items as $key => $value ) {
				$index++;
				if ( 'customer-logout' === $key ) {
					break;
				}
			}

			$items = array_slice( $items, 0, $index ) +
				array( $account_slug => nmgr_get_account_details( 'name' ) ) +
				array_slice( $items, $index );
		} elseif ( $account_slug ) {
			$items[ $account_slug ] = nmgr_get_account_details( 'name' );
		}

		return apply_filters( 'nmgr_account_menu_item', $items );
	}

	/**
	 * NM Gift Registry endpoint title on the woocommerce my account page
	 */
	public static function endpoint_title( $title, $endpoint ) {
		$account_slug = nmgr_get_account_details( 'slug' );
		if ( $account_slug && ($account_slug == $endpoint) ) {
			return apply_filters( 'nmgr_endpoint_title', nmgr_get_account_details( 'name' ) );
		}
		return $title;
	}

	/**
	 * Template for managing wishlists on the woocommerce my account page
	 *
	 * @param string $wishlist_slug  Slug of the wishlist currently being viewed (Pro version)
	 */
	public static function account_endpoint() {
		function_exists( 'wc_print_notice' ) ? wc_print_notice( nmgr_get_relocation_notice(), 'notice' ) : '';
	}

	/**
	 * Filter the returned values for data property
	 *
	 * @param mixed $value The value
	 * @param string $prop The property
	 * @param string $parent The parent of the property
	 * @param Object $object The type of data to modify. Valid values are 'wishlist' and 'wishlist_item'
	 */
	public static function get_wishlist_property( $value, $prop, $parent, $object ) {
		if ( 'wishlist' === $object->get_object_type() && 'gift-registry' === $object->get_type() ) {
			$type = $object->get_type();

			$dont_modify = array( 'post_status' );

			if ( !in_array( $prop, $dont_modify ) && 'no' === nmgr_get_type_option( $type, "display_form_{$prop}" ) ) {
				return null;
			}

			// Images
			$images = array(
				'background' => 'background_image_id',
				'thumbnail' => 'thumbnail_id'
			);

			if ( in_array( $prop, $images ) ) {
				$key = array_search( $prop, $images );
				if ( 'no' === nmgr_get_type_option( $type, "display_image_{$key}" ) ) {
					return null;
				}
			}

			// Shipping
			if ( !nmgr_get_type_option( $type, 'enable_shipping', 1 ) ) {
				if ( 'shipping' === $parent ) {
					return null;
				}

				if ( 'shipping' === $prop ) {
					return array();
				}
			}
		}
		return $value;
	}

	/**
	 * Show the theme sidebar if it exists
	 * (checks only for sidebar.php or siderbar-shop.php)
	 * @deprecated since version 4.0.0
	 */
	public static function show_theme_sidebar() {
		$templates = array(
			'sidebar-shop.php',
			'sidebar.php'
		);

		$file_exists = false;

		foreach ( $templates as $template_name ) {
			if ( file_exists( get_stylesheet_directory() . '/' . $template_name ) ||
				file_exists( get_template_directory() . '/' . $template_name ) ) {
				$file_exists = true;
				break;
			}
		}

		if ( $file_exists ) {
			get_sidebar( 'shop' );
		}
	}

	/**
	 * Show notice when there are no wishlist search results
	 * @since 2.2.0
	 */
	public static function no_search_results_notification() {
		if ( !is_nmgr_search() ) {
			return;
		}

		echo '<p class="woocommerce-info">' .
		sprintf(
			/* translators: %s: wishlist type title */
			esc_html( nmgr()->is_pro ? __( 'No %s were found matching your selection.', 'nm-gift-registry' ) : __( 'No %s were found matching your selection.', 'nm-gift-registry-lite' )  ),
			esc_html( nmgr_get_type_title( '', true ) )
		) .
		'</p>';
	}

	public static function notify_of_item_purchased_status( $notice, $item ) {
		if ( $item->get_purchased_quantity() ) {
			$notice .= ' ' . (nmgr()->is_pro ?
				__( 'This item has purchases that may be lost if deleted.', 'nm-gift-registry' ) :
				__( 'This item has purchases that may be lost if deleted.', 'nm-gift-registry-lite' ));
		}
		return $notice;
	}

	public static function show_archive_header( $query, $args ) {
		if ( $args[ 'show_header' ] ?? true ) :
			?>
			<header class="nmgr-archive-header nmgr-text-center">
				<?php
				if ( $args[ 'show_title' ] ?? true ) :
					if ( !empty( $args[ 'title' ] ) ) {
						$title = $args[ 'title' ];
					} elseif ( is_nmgr_search( $query ) ) {
						$title = sprintf(
							/* translators: %s: search query */
							nmgr()->is_pro ? __( 'Search results for: &ldquo;%s&rdquo;', 'nm-gift-registry' ) : __( 'Search results for: &ldquo;%s&rdquo;', 'nm-gift-registry-lite' ),
							nmgr_get_search_query()
						);
					} elseif ( $query->is_tax() ) {
						$title = single_term_title( '', false );
					} elseif ( $query->is_post_type_archive( 'nm_gift_registry' ) ) {
						$title = nmgr_get_type_title( 'c', true );
					} else {
						$title = get_the_archive_title();
					}

					$title = apply_filters( 'nmgr_archive_title', $title, $query, $args );

					if ( $title ) :
						?>
						<h2 class="nmgr-archive-title">
							<?php echo wp_kses_post( $title ); ?>
						</h2>
						<?php
					endif;
				endif;

				do_action( 'nmgr_after_archive_title', $query, $args );
				?>
			</header>
			<?php
		endif;
	}

	public static function show_archive_results_count_and_page_number( $query, $args ) {
		$show_results_count = $args[ 'show_results_count' ] ?? is_nmgr_archive();
		$show_on_page = apply_filters( 'nmgr_show_archive_results_count', $show_results_count );

		if ( $show_on_page ) :
			?>
			<p class="nmgr-archive-results-metadata">
				<?php
				$results_found = '<span class="results-found">' . sprintf(
						/* translators: %d: total results */
						nmgr()->is_pro ? _n( '%d result found', '%d results found', $query->found_posts, 'nm-gift-registry' ) : _n( '%d result found', '%d results found', $query->found_posts, 'nm-gift-registry-lite' ),
						( int ) $query->found_posts
					) . '</span>';

				if ( get_query_var( 'paged' ) ) {
					$results_found .= '<span class="page-number">' . sprintf(
							/* translators: %s: page number */
							nmgr()->is_pro ? __( '&nbsp;&ndash; Page %s', 'nm-gift-registry' ) : __( '&nbsp;&ndash; Page %s', 'nm-gift-registry-lite' ),
							get_query_var( 'paged' )
						) . '</span>';
				}
				echo $results_found;
				?>
			</p>
			<?php
		endif;
	}

	public static function set_toast() {
		$comp = nmgr_get_toast();
		$comp->wrapper_attributes[ 'class' ][] = 'nmgr';
		$comp->get_wrapper( true );
	}

}
