<?php
/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

class NMGR_Widget_Cart extends WP_Widget {

	public function __construct() {
		parent::__construct(
			'nmgr_cart',
			nmgr()->is_pro ?
				__( 'NM Gift Registry Cart', 'nm-gift-registry' ) :
				__( 'NM Gift Registry Cart', 'nm-gift-registry-lite' ),
			array(
				'description' => nmgr()->is_pro ?
					__( 'Display a user\'s wishlists like a cart.', 'nm-gift-registry' ) :
					__( 'Display a user\'s wishlists like a cart.', 'nm-gift-registry-lite' ),
			)
		);

		add_action( 'widgets_init', function() {
			register_widget( 'NMGR_Widget_Cart' );
		} );
	}

	protected function get_form_fields() {
		return array(
			'type' => array(
				'type' => 'select',
				'label' => nmgr()->is_pro ?
				__( 'Wishlist type', 'nm-gift-registry' ) :
				__( 'Wishlist type', 'nm-gift-registry-lite' ),
				'options' => [
					'gift-registry' => nmgr()->is_pro ?
					__( 'Gift Registry', 'nm-gift-registry' ) :
					__( 'Gift Registry', 'nm-gift-registry-lite' ),
					'wishlist' => nmgr()->is_pro ?
					__( 'Wishlist', 'nm-gift-registry' ) :
					__( 'Wishlist', 'nm-gift-registry-lite' ),
				],
			),
		);
	}

	public function widget( $args, $instance ) {
		$template_args = array(
			'type' => $instance[ 'type' ] ?? 'gift-registry',
		);

		if ( !is_nmgr_enabled( $template_args[ 'type' ] ) ) {
			return;
		}

		echo $args[ 'before_widget' ];
		echo '<div class="widget_nmgr_cart_content">';
		nmgr_get_cart_template( $template_args, true );
		echo '</div>';
		echo $args[ 'after_widget' ];
	}

	public function form( $instance ) {
		$fields = $this->get_form_fields();

		if ( empty( $fields ) ) {
			return;
		}

		foreach ( $fields as $key => $setting ) {
			$class = isset( $setting[ 'class' ] ) ? $setting[ 'class' ] : '';
			$value = isset( $instance[ $key ] ) ? $instance[ $key ] : (isset( $setting[ 'std' ] ) ? $setting[ 'std' ] : '');

			switch ( $setting[ 'type' ] ) {
				case 'text':
				case 'number':
					?>
					<p>
						<label for="<?php echo esc_attr( $this->get_field_id( $key ) ); ?>"><?php echo wp_kses_post( $setting[ 'label' ] ); ?></label>
						<input class="widefat <?php echo esc_attr( $class ); ?>" id="<?php echo esc_attr( $this->get_field_id( $key ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $key ) ); ?>" type="<?php echo $setting[ 'type' ]; ?>" value="<?php echo esc_attr( $value ); ?>" />
					</p>
					<?php
					break;

				case 'select':
					?>
					<p>
						<label for="<?php echo esc_attr( $this->get_field_id( $key ) ); ?>"><?php echo $setting[ 'label' ]; /* phpcs:ignore WordPress.XSS.EscapeOutput.OutputNotEscaped */ ?></label>
						<select class="widefat <?php echo esc_attr( $class ); ?>" id="<?php echo esc_attr( $this->get_field_id( $key ) ); ?>" name="<?php echo esc_attr( $this->get_field_name( $key ) ); ?>">
							<?php foreach ( $setting[ 'options' ] as $option_key => $option_value ) : ?>
								<option value="<?php echo esc_attr( $option_key ); ?>" <?php selected( $option_key, $value ); ?>><?php echo esc_html( $option_value ); ?></option>
							<?php endforeach; ?>
						</select>
					</p>
					<?php
					break;
			}
		}
	}

	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		if ( empty( $this->get_form_fields() ) ) {
			return $instance;
		}

		foreach ( $this->get_form_fields() as $key => $setting ) {
			if ( !isset( $setting[ 'type' ] ) ) {
				continue;
			}

			switch ( $setting[ 'type' ] ) {
				default:
					$instance[ $key ] = isset( $new_instance[ $key ] ) ? sanitize_text_field( $new_instance[ $key ] ) : $setting[ 'std' ];
					break;
			}
		}

		return $instance;
	}

}
