<?php
/**
 * Sync
 */
defined( 'ABSPATH' ) || exit;

use \NmeriMedia\V2\Bootstrap\Dropdown;

class NMGR_Wishlist_Template {

	private $type;

	/**
	 * @var NMGR_Wishlist
	 */
	private $wishlist;
	private $notices = [];
	private $is_disabled;
	private $is_password_protected;

	public function __construct( $id = false ) {
		$this->wishlist = nmgr_get_wishlist( $id, true );
		$this->type = $this->wishlist ? $this->wishlist->get_type() : nmgr_get_current_type();
		$this->is_password_protected = $this->wishlist ? post_password_required( $this->wishlist->get_id() ) : false;
	}

	public function get_id() {
		return $this->wishlist ? $this->wishlist->get_id() : 0;
	}

	public function set_type( $type ) {
		$this->type = $type;
	}

	private function set_notice( $key, $message, $type = 'notice' ) {
		$this->notices[ $key ] = [
			'message' => $message,
			'type' => $type,
		];
	}

	public function process_notices() {
		$is_single = is_nmgr_wishlist();

		if ( !$this->wishlist || ($is_single && ($this->is_disabled || $this->is_password_protected)) ) {
			return;
		}

		$is_gift_registry = $this->wishlist->is_type( 'gift-registry' );
		$user_has_wishlist = nmgr_user_has_wishlist( $this->wishlist );
		$is_archived = is_callable( [ $this->wishlist, 'is_archived' ] ) ? $this->wishlist->is_archived() : false;
		$is_admin = is_nmgr_admin();
		$single_or_admin = $is_single || $is_admin;

		if ( $is_gift_registry && $is_archived && ($single_or_admin || is_nmgr_account_section()) ) {
			$this->set_notice( 'wishlist_archived', $this->get_registered_notice( 'wishlist_archived' ) );
		}

		if ( $is_gift_registry && ($single_or_admin || is_nmgr_account_section( 'shipping' )) &&
			!$is_archived && $this->wishlist->needs_shipping_address() ) {
			$notice = $this->get_registered_notice( 'require_shipping_address' );

			if ( $is_single && $user_has_wishlist ) {
				$link = trailingslashit( $this->wishlist->get_permalink() ) . 'shipping';
				$filtered_link = apply_filters( 'nmgr_set_shipping_address_url', $link, $this->wishlist );

				if ( $filtered_link ) {
					$notice .= sprintf( '<a class="button" href="%s">%s</a>',
						$filtered_link,
						nmgr()->is_pro ?
						__( 'Set now', 'nm-gift-registry' ) :
						__( 'Set now', 'nm-gift-registry-lite' )
					);
				}
			}
			$this->set_notice( 'require_shipping_address', $notice );
		}

		if ( $is_gift_registry && $single_or_admin && $this->wishlist->is_fulfilled() ) {
			$this->set_notice( 'wishlist_fulfilled', $this->get_registered_notice( 'wishlist_fulfilled' ) );
		}

		if ( $is_single && !$is_archived && !$this->wishlist->has_items() ) {
			$link = '';
			if ( $user_has_wishlist ) {
				$link = sprintf( '<a href="%s" tabindex="1" class="button">%s</a>',
					nmgr_get_add_items_url(),
					nmgr()->is_pro ?
					__( 'Add item(s)', 'nm-gift-registry' ) :
					__( 'Add item(s)', 'nm-gift-registry-lite' )
				);
			}

			$this->set_notice( 'wishlist_empty', $link . $this->get_registered_notice( 'wishlist_empty' ) );
		}
	}

	private function get_body_html() {
		ob_start();
		if ( is_nmgr_wishlist_page( 'new' ) ) {
			if ( is_nmgr_user( $this->type ) ) {
				$text = nmgr()->is_pro ?
					__( 'Add new', 'nm-gift-registry' ) :
					__( 'Add new', 'nm-gift-registry-lite' );
				echo '<h3 class="nmgr-add-new-header-text">' . esc_html( $text ) . '</h3>';

				nmgr()->sub()->show_new_wishlist_template( $this->type );
			} else {
				$this->set_login_notice();
			}
		} elseif ( is_nmgr_wishlist_page( 'account_section' ) ) {
			if ( nmgr_user_has_wishlist( $this->wishlist ) ) {
				$section = get_query_var( 'nmgr_action' );
				$title = nmgr()->account( $this->wishlist )->set_section( $section )->get_section_title();

				if ( $title ) :
					?>
					<div class="nmgr-account-section-header">
						<a href="<?php echo esc_url( $this->wishlist->get_permalink() ); ?>"
							 class="nmgr-wishlist-title">
								 <?php echo esc_html( $this->wishlist->get_title() ); ?>
						</a>
						<h3 class="nmgr-template-title <?php echo esc_attr( $section ); ?>">
							<?php echo wp_kses_post( $title ); ?>
						</h3>
					</div>
					<?php
				endif;
				echo nmgr_get_account_section( $section, $this->wishlist->get_id() );
			}
		} elseif ( is_nmgr_wishlist_page( 'home' ) ) {
			if ( is_nmgr_user( $this->type ) ) {
				if ( !nmgr_get_user_wishlists_count( '', $this->type ) ) {
					echo nmgr_get_default_account_section_content( "create_{$this->type}" );
				} else {
					nmgr()->sub()->show_all_wishlists_template( $this->type );
				}
			} else {
				$this->set_login_notice();
			}
		} elseif ( is_nmgr_wishlist() ) {
			if ( is_nmgr_wishlist_page( 'base' ) ) {
				if ( is_nmgr_user( $this->type ) ) {
					if ( !nmgr_get_user_wishlists_count( '', $this->type ) ) {
						echo nmgr_get_default_account_section_content( "create_{$this->type}" );
					} elseif ( nmgr_user_has_wishlist( $this->wishlist ) ) {
						echo $this->get_post();
					}
				} else {
					$this->set_login_notice();
				}
			} else {
				if ( !$this->wishlist ) {
					$this->is_disabled = true;
					$this->set_notice( 'invalid_wishlist', $this->get_registered_notice( 'invalid_wishlist' ) );
				} elseif ( 'private' === $this->wishlist->get_status() && !nmgr_user_has_wishlist( $this->wishlist ) ) {
					$this->is_disabled = true;
					$this->set_notice( 'private_wishlist', $this->get_registered_notice( 'private_wishlist' ) );
				} else {
					echo $this->get_post();
				}
			}
		}
		return ob_get_clean();
	}

	private function get_post() {
		if ( $this->is_disabled ) {
			return;
		}

		$query_args = array(
			'posts_per_page' => 1,
			'post_type' => 'nm_gift_registry',
			'post_status' => nmgr_get_post_statuses(),
			'post__in' => [ $this->get_id() ],
		);

		$single = new WP_Query( $query_args );

		ob_start();

		if ( $single->have_posts() ) {
			while ( $single->have_posts() ) {
				$single->the_post();

				nmgr_template( 'content-single-nm_gift_registry.php', [ 'wishlist' => $this->wishlist ] );
			}

			wp_reset_postdata();
		}

		return ob_get_clean();
	}

	private function set_login_notice() {
		$url = add_query_arg( array(
			'nmgr-notice' => 'login-to-access',
			'nmgr-redirect' => $_SERVER[ 'REQUEST_URI' ],
			'nmgr-type' => $this->type,
			), wc_get_page_permalink( 'myaccount' ) );

		$notice = $this->get_registered_notice( 'login-to-access' ) . ' ' . nmgr_get_click_here_link( $url );

		$this->set_notice( 'login-to-access', $notice );
	}

	public function get_registered_notice( $key ) {
		$notice = '';

		switch ( $key ) {
			case 'login-to-access':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					nmgr()->is_pro ? __( 'Login to access your %s.', 'nm-gift-registry' ) : __( 'Login to access your %s.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title( '', false, $this->type )
				);
				break;

			case 'invalid_wishlist':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					nmgr()->is_pro ? __( 'This %s does not exist or it cannot be viewed.', 'nm-gift-registry' ) : __( 'This %s does not exist or it cannot be viewed.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title( '', false, $this->type )
				);
				break;

			case 'private_wishlist':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					__( 'This %s is private.', 'nm-gift-registry' ),
					nmgr_get_type_title( '', false, $this->type )
				);
				break;

			case 'wishlist_archived':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					__( 'This %s is archived.', 'nm-gift-registry' ),
					nmgr_get_type_title()
				);
				break;

			case 'wishlist_fulfilled':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					nmgr()->is_pro ? __( 'This %s is fulfilled.', 'nm-gift-registry' ) : __( 'This %s is fulfilled.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title( '', false, $this->type )
				);
				break;

			case 'wishlist_empty':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					nmgr()->is_pro ? __( 'This %s is empty.', 'nm-gift-registry' ) : __( 'This %s is empty.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title( '', false, $this->type )
				);
				break;

			case 'require_shipping_address':
				$notice = sprintf(
					/* translators: %s: wishlist type title */
					nmgr()->is_pro ? __( 'The shipping address for this %s must be properly filled before items can be purchased from it.', 'nm-gift-registry' ) : __( 'The shipping address for this %s must be properly filled before items can be purchased from it.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title()
				);
				break;
		}

		return $notice;
	}

	public function get_notices() {
		return $this->notices;
	}

	private function get_notices_html() {
		ob_start();

		$this->process_notices();

		foreach ( $this->notices as $notice ) {
			if ( function_exists( 'wc_print_notice' ) ) {
				wc_print_notice( $notice[ 'message' ], 'notice' );
			}
		}
		return ob_get_clean();
	}

	private function process_menu() {
		$wishlists_count = nmgr_get_user_wishlists_count( nmgr_get_current_user_id(), $this->type );
		$user_has_wishlist = nmgr_user_has_wishlist( $this->wishlist );

		if ( !is_nmgr_user( $this->type ) || $this->is_disabled || ($this->wishlist && !$user_has_wishlist) ) {
			return [];
		}

		$is_home = is_nmgr_wishlist_page( 'home' );
		$is_new = is_nmgr_wishlist_page( 'new' );
		$is_single = is_nmgr_wishlist();
		$is_archived = is_callable( [ $this->wishlist, 'is_archived' ] ) ? $this->wishlist->is_archived() : false;
		$allow_multi = nmgr()->is_pro ? nmgr_get_type_option( $this->type, 'allow_multiple_wishlists' ) : false;

		$actions = [
			'manage' => [
				'text' => '',
				'priority' => 10,
				'attributes' => [
					'class' => [
						'nmgr-wishlist-edit-link',
					],
					'href' => '',
				],
				'show' => $user_has_wishlist,
			],
			'add_new' => [
				'text' => nmgr()->is_pro ?
				__( 'Add new', 'nm-gift-registry' ) :
				__( 'Add new', 'nm-gift-registry-lite' ),
				'priority' => 20,
				'attributes' => [
					'href' => nmgr_get_url( $this->type, 'new' ),
				],
				'show' => ($user_has_wishlist || $is_home || $is_new || $is_single) && (!$wishlists_count || $allow_multi),
			],
			'see_all' => [
				'text' => __( 'See all', 'nm-gift-registry' ),
				'priority' => 30,
				'attributes' => [
					'href' => nmgr_get_url( $this->type, 'home' ),
				],
				'show' => ($is_new && $wishlists_count) ||
				(($user_has_wishlist || $is_home) && $wishlists_count && $allow_multi),
			],
			'add_items' => [
				'text' => nmgr()->is_pro ?
				__( 'Add item(s)', 'nm-gift-registry' ) :
				__( 'Add item(s)', 'nm-gift-registry-lite' ),
				'priority' => 40,
				'attributes' => [
					'class' => [
						'nmgr-tip',
					],
					'title' => nmgr_get_add_items_text(),
					'href' => nmgr_get_add_items_url(),
				],
				'show' => ($user_has_wishlist && !$this->wishlist->needs_shipping_address() && !$is_archived) ||
				$is_single || $is_home,
			],
			'view' => [
				'text' => nmgr()->is_pro ?
				__( 'View', 'nm-gift-registry' ) :
				__( 'View', 'nm-gift-registry-lite' ),
				'priority' => 50,
				'attributes' => [
					'href' => $this->wishlist ? $this->wishlist->get_permalink() : '#',
				],
				'show' => $user_has_wishlist && is_nmgr_wishlist_page( 'account_section' ),
			],
		];

		foreach ( array_keys( $actions ) as $key ) {
			$actions[ $key ][ 'attributes' ][ 'class' ][] = 'nmgr-menu-item';
		}

		$account = nmgr()->account( $this->wishlist );
		$account_sections = $account->get_sections_data();
		$dropdown = new Dropdown();
		$dropdown->set_text( nmgr()->is_pro ?
				__( 'Manage', 'nm-gift-registry' ) :
				__( 'Manage', 'nm-gift-registry-lite' )  );

		$dropdown->toggle_attributes = nmgr()->merge_args( $actions[ 'manage' ][ 'attributes' ], $dropdown->toggle_attributes );

		foreach ( $account_sections as $dkey => $daction ) {
			if ( in_array( $dkey, [ 'items', 'images' ] ) ) {
				continue;
			}

			$attributes = [
				'href' => $this->wishlist ? trailingslashit( $this->wishlist->get_permalink() ) . $dkey : '#',
			];

			if ( is_nmgr_account_section( $dkey ) ) {
				$attributes[ 'class' ] = [ 'active' ];
			}

			$dropdown->set_menu_item(
				($daction[ 'title' ] ?? '' ),
				($attributes ),
				$dkey
			);
		}

		$manage_text = $dropdown->get();
		$actions[ 'manage' ][ 'text' ] = $manage_text;

		$factions = apply_filters( 'nmgr_wishlist_menu', $actions, $this->get_id() );
		$showing_actions = nmgr_get_elements_to_show( $factions );
		uasort( $showing_actions, 'nmgr_sort_by_priority' );

		return $showing_actions;
	}

	private function get_menu_html() {
		ob_start();
		$menu = $this->process_menu();

		if ( !empty( $menu ) ) :
			?>
			<div class="nmgr-dashboard">
				<div class="nmgr-wishlist-menu">
					<?php
					foreach ( $menu as $action ) :
						if ( !empty( $action[ 'attributes' ][ 'href' ] ) ) :
							?>
							<a <?php echo nmgr()->format_attributes( $action[ 'attributes' ] ); ?>>
								<?php echo wp_kses( $action[ 'text' ], nmgr()->allowed_post_tags() ); ?>
							</a>
							<?php
						elseif ( !empty( $action[ 'text' ] ) ) :
							echo wp_kses( $action[ 'text' ], nmgr()->allowed_post_tags() );
						endif;
					endforeach;
					?>
				</div>

				<?php
				if ( nmgr_user_has_wishlist( $this->wishlist ) && is_nmgr_wishlist() ) :
					?>
					<div class="nmgr-dashboard-right">
						<?php
						if ( $this->wishlist->is_type( 'gift-registry' ) ) {
							$this->show_messages_icon( $this->wishlist );
						}

						$this->show_status_icon( $this->wishlist );
						?>
					</div>
				<?php endif; ?>

			</div>
			<?php
		endif;

		return ob_get_clean();
	}

	private function show_messages_icon() {
		$account = nmgr()->account( $this->wishlist )->set_section( 'messages' );
		if ( !empty( $account->get_section_data() ) ) {
			$messages = $account->get_messages();
			$messages_count = count( $messages );
			$title = sprintf(
				/* translators: %d: messages count */
				_n(
					'%d message is in your inbox',
					'%d messages are in your inbox',
					$messages_count,
					'nm-gift-registry' ),
				$messages_count
			);

			if ( $messages_count ) :
				?>
				<a class="nmgr-messages-icon nmgr-tip" title="<?php esc_attr_e( $title ); ?>"
					 href="<?php echo esc_url( $this->wishlist->get_permalink() . '/messages' ); ?>">
					<div class="hang-badge">
						<?php
						$icon_args = [
							'icon' => 'bubble',
							'size' => '1.2',
							'fill' => '#eee',
						];

						echo wp_kses( nmgr_get_svg( $icon_args ), nmgr()->allowed_svg_tags() );
						?>

						<?php if ( $messages_count ) : ?>
							<span class="nm-component badge" style="background-color:#ccc;">
								<?php echo ( int ) $messages_count; ?>
							</span>
						<?php endif; ?>
					</div>
				</a>
				<?php
			endif;
		}
	}

	private function show_status_icon() {
		$html = '';

		if ( !$this->wishlist ) {
			return;
		}

		if ( method_exists( $this->wishlist, 'get_password' ) && !empty( $this->wishlist->get_password() ) ) {
			$html = nmgr_get_svg( array(
				'icon' => 'lock',
				'class' => 'nmgr-tip',
				'fill' => '#999',
				'size' => '1.2',
				'sprite' => false,
				'title' => sprintf(
					/* translators: %s: wishlist type title */
					__( 'Your %s is password protected. Do not forget to share the password with your guests.', 'nm-gift-registry' ),
					nmgr_get_type_title() ),
				) );
		} elseif ( method_exists( $this->wishlist, 'get_status' ) && 'private' === $this->wishlist->get_status() ) {
			$html = nmgr_get_svg( array(
				'icon' => 'eye-cancel',
				'class' => 'nmgr-tip',
				'fill' => '#999',
				'size' => '1.2',
				'sprite' => false,
				'title' => sprintf(
					/* translators: %s: wishlist type title */
					__( 'Your %s is published privately. This means that only you can see it when logged in.', 'nm-gift-registry' ),
					nmgr_get_type_title() ),
				) );
		}

		if ( $html ) {
			?>
			<div class="nmgr-wishlist-status"><?php echo wp_kses( $html, nmgr()->allowed_svg_tags() ); ?></div>
			<?php
		}
	}

	public function get() {
		if ( !is_nmgr_enabled( $this->type ) ) {
			return;
		}

		$body = $this->get_body_html();
		$menu = $this->get_menu_html();
		$notices = $this->get_notices_html(); //always get notices last

		$template_full = '<div class="woocommerce nmgr-template">' . $notices . $menu . $body . '</div>';
		return apply_filters( 'nmgr_wishlist_template', $template_full, $this );
	}

}
