<?php

namespace NmeriMedia\V2\Traits;

defined( 'ABSPATH' ) || exit;

/**
 * The properties of the plugin
 * @class \NmeriMedia\V2\Traits\PluginProps
 */
trait PluginProps {

	public $name;
	public $version;
	public $url;
	public $path;

	/**
	 * Plugin root filepath e.g /www/html/wordpress/wp-content/plugins/nm-plugin/nm-plugin.php
	 */
	public $file;

	/**
	 * Slug of plugin root file slug e.g nm-plugin
	 */
	public $slug;

	/**
	 * Basename of plugin root file e.g. nm-plugin/nm-plugin.php
	 */
	public $basename;

	/**
	 * Plugin base e.g nm_plugin
	 * Usually taken from plugin root file
	 */
	public $base;

	/**
	 * Whether the plugin is licensed
	 * @var boolean
	 */
	public $is_licensed = false;

	/**
	 * Pro version
	 * @var boolean
	 */
	public $is_pro = false;

	public function set_plugin_props( $filepath = null ) {
		$filedata = [];

		if ( !is_null( $filepath ) ) {
			if ( !function_exists( 'get_plugin_data' ) ) {
				require_once ABSPATH . 'wp-admin/includes/plugin.php';
			}

			$this->file = $filepath;
			$this->url = plugin_dir_url( $filepath );
			$this->path = plugin_dir_path( $filepath );
			$this->slug = pathinfo( $filepath, PATHINFO_FILENAME );
			$this->basename = plugin_basename( $filepath );
			$this->base = str_replace( '-', '_', $this->slug );

			$plugin_headers = array(
				'name' => 'Plugin Name',
				'version' => 'Version',
			);

			$filedata = get_file_data( $filepath, $plugin_headers );

			foreach ( $filedata as $key => $val ) {
				if ( property_exists( $this, $key ) ) {
					$this->{$key} = $val;
				}
			}
		}
	}

}
