<?php
/**
 * Template for displaying items in a wishlist
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.2.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;

global $post;
?>

<div <?php echo nmgr_format_attributes( $attributes ); ?>>
	<?php
	if ( !$wishlist ):

		echo nmgr_get_default_account_section_content( 'items', $wishlist );

	else :

		$is_archived = is_callable( [ $wishlist, 'is_archived' ] ) ? $wishlist->is_archived() : false;
		?>

		<fieldset <?php echo $is_archived ? 'disabled' : ''; ?>>

			<?php
			$display_mode = $view->get_display()[ 'mode' ];
			$view_container_attributes = $view->get_items_container_attributes();
			$parts_data = $view->get_parts_data();
			$parts_keys = array_keys( $parts_data );
			$item_count = $wishlist->get_items_quantity_count();

			do_action( 'nmgr_before_items', $items, $wishlist, $view->get_args(), $view );

			if ( $item_count ) :
				?>
				<div class="nmgr-items-menu">
					<?php
					if ( is_callable( [ $view, 'show_items_bulk_actions' ] ) ) {
						$view->show_items_bulk_actions();
					}

					if ( !empty( $pagination ) ) {
						$pagination[ 'position' ] = 'top';
						echo nmgr_get_nav( $pagination );
					}

					if ( is_callable( [ $view, 'show_items_display_modes_toggle' ] ) ) {
						$view->show_items_display_modes_toggle();
					}
					?>
				</div>
				<?php
			endif;
			?>

			<?php
			if ( !is_nmgr_admin() && nmgr_user_has_wishlist( $wishlist ) &&
				$wishlist->is_type( 'gift-registry' ) && $item_count &&
				nmgr_get_type_option( 'gift-registry', 'display_item_quantity' ) &&
				nmgr_get_type_option( 'gift-registry', 'display_item_purchased_quantity' ) ) :
				?>
				<div class="nmgr-items-count-progress">
					<div class="nmgr-progressbar-wrapper">
						<?php
						$item_purchased_count = $wishlist->get_items_purchased_quantity_count();
						$title_attribute = sprintf(
							/* translators: 1: item quantity purchased, 2: item quantity in wishlist */
							nmgr()->is_pro ? __( '%1$s of %2$s items purchased.', 'nm-gift-registry' ) : __( '%1$s of %2$s items purchased.', 'nm-gift-registry-lite' ),
							$item_purchased_count,
							$item_count
						);
						echo wp_kses( nmgr_progressbar( $item_count, $item_purchased_count, $title_attribute ),
							array_merge( wp_kses_allowed_html( 'post' ), nmgr_allowed_svg_tags() ) );
						?>
					</div>
					<div class="nmgr-item-count">
						<?php
						printf(
							/* translators: %d: item count */
							nmgr()->is_pro ? _nx( '%d item', '%d items', $item_count, 'wishlist item count', 'nm-gift-registry' ) : _nx( '%d item', '%d items', $item_count, 'wishlist item count', 'nm-gift-registry-lite' ),
							$item_count
						);
						?>
					</div>

				</div>
				<?php
			endif;
			?>

			<?php
			if ( 'list' === $display_mode ) {
				echo "<table $view_container_attributes><thead><tr>";
				foreach ( $parts_keys as $part ) {
					echo $view->get_table_header( $part );
				}
				echo '</tr></thead><tbody>';
			} else {
				echo "<div $view_container_attributes>";
			}

			foreach ( $items as $item_id => $item ) :
				$the_product = $item->get_product();
				$view->set_item( $item );

				if ( !$the_product ) {
					if ( !$view->is_public() ) {
						echo ('list' === $display_mode ? '<tr ' : '<div ')
						. 'class="item-deleted-notice">'
						. ('list' === $display_mode ? '<td colspan="20">' : '<div>')
						. sprintf(
							/* translators : %s wishlist type title */
							nmgr()->is_pro ? __( 'An item has been deleted from your %s as it no longer exists', 'nm-gift-registry' ) : __( 'An item has been deleted from your %s as it no longer exists', 'nm-gift-registry-lite' ),
							esc_html( nmgr_get_type_title() )
						)
						. ('list' === $display_mode ? '</td></tr>' : '</div></div>');
						$item->delete();
					}
					continue;
				}

				if ( $view->item_is_disabled() ) {
					continue;
				}

				echo ('list' === $display_mode ? '<tr ' : '<div ') . $view->get_item_container_attributes() . '">';
				wc_setup_product_data( $the_product->get_id() );

				foreach ( $parts_data as $part => $part_data ) {
					if ( isset( $part_data[ 'content' ] ) ) {
						echo $view->get_item_part_content( $part );
					} elseif ( method_exists( $view, 'get_part' ) ) {
						echo $view->get_part( $part );
					}
				}

				wc_setup_product_data( $post );
				echo 'list' === $display_mode ? '</tr>' : '</div>';

			endforeach;

			echo 'list' === $display_mode ? '</tbody>' : '';
			do_action( 'nmgr_after_items_table_body', $items, $wishlist, $view->get_args() );
			echo 'list' === $display_mode ? '</table>' : '</div>';

			if ( !empty( $pagination ) ) {
				$pagination[ 'position' ] = 'bottom';
				echo nmgr_get_nav( $pagination );
			}

			do_action( 'nmgr_after_items', $items, $wishlist, $view->get_args(), $view );
			?>

		</fieldset>

	<?php
	endif;
	?>
</div>
