<?php
/**
 * Wishlist item add to cart action
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.2.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;
?>
<div>
	<?php
	do_action( 'nmgr_item_before_add_to_cart_form', $item );

	$item_statuses = $view->get_item_statuses_for_disabling_add_to_cart();

	if ( !empty( $item_statuses ) ) :
		foreach ( $item_statuses as $status_key => $status_args ) {
			$attributes = nmgr()->flatten_attributes( $status_args[ 'add_to_cart_column_attributes' ] ?? [] );
			?>
			<div class="<?php echo esc_attr( $status_key ); ?>">
				<span <?php echo $attributes; ?>>
					<?php echo wp_kses_post( $status_args[ 'label' ], nmgr()->allowed_post_tags() ); ?>
				</span>
			</div>
			<?php
		}

	else:

		$add_to_cart_form = apply_filters( 'nmgr_add_to_cart_form', null, $args );

		if ( $add_to_cart_form ) :

			echo $add_to_cart_form;

		else :
			?>

			<form class="cart nmgr-add-to-cart-form"
						action="<?php the_permalink(); ?>"
						method="post"
						enctype='multipart/form-data'>

				<input type="hidden" name="nmgr-add-to-cart-product-id" value="<?php echo ( int ) $item->get_product_id(); ?>" />
				<input type="hidden" name="nmgr-add-to-cart-wishlist-item" value="<?php echo absint( $item->get_id() ); ?>" />
				<input type="hidden" name="nmgr-add-to-cart-wishlist" value="<?php echo absint( $wishlist->get_id() ); ?>" />

				<?php if ( 'variation' === $product->get_type() ) : ?>
					<input type="hidden" name="variation_id" value="<?php echo absint( $item->get_variation_id() ); ?>" />
					<?php
					if ( !empty( $item->get_variation() ) ) :
						foreach ( $item->get_variation() as $attribute_key => $value ) :
							?>
							<input type="hidden" name="<?php echo esc_attr( $attribute_key ); ?>" value="<?php echo esc_attr( $value ); ?>" />
							<?php
						endforeach;
					endif;
				endif;
				?>

				<?php
				do_action( 'nmgr_item_add_to_cart_form', $item );

				woocommerce_quantity_input( array(
					'min_value' => $product->get_min_purchase_quantity(),
					'max_value' => $max_qty,
					), $product );
				?>

				<button type="submit"
								class="<?php echo implode( ' ', array_filter( array_unique( $button_classes ) ) ); ?>"
								data-product_id="<?php echo absint( $product->get_id() ); ?>"
								data-wishlist_item_id="<?php echo absint( $item->get_id() ); ?>"
								data-wishlist_id="<?php echo absint( $wishlist->get_id() ); ?>">
									<?php echo esc_html( $product->single_add_to_cart_text() ); ?>
				</button>
			</form>

		<?php
		endif;
	endif;
	do_action( 'nmgr_item_after_add_to_cart_form', $item );
	?>

</div>
