<?php
/**
 * Wishlist messages
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.1.0
 */
defined( 'ABSPATH' ) || exit;
?>

<div <?php echo nmgr_format_attributes( $attributes ); ?>>

	<?php
	if ( !$wishlist || empty( $messages ) ):

		if ( !$wishlist ) :
			echo nmgr_get_default_account_section_content( 'messages', $wishlist );
		else :
			if ( empty( $messages ) && has_action( 'nmgr_no_messages' ) ) :
				do_action_deprecated( 'nmgr_no_messages', array( $wishlist, $messages_args ), '2.2.1', '', __( 'Use the filters and actions in \'nmgr_get_default_account_section_content\' instead', 'nm-gift-registry' ) );
			elseif ( empty( $messages ) ) :
				echo nmgr_get_default_account_section_content( 'messages', $wishlist );
			endif;
		endif;

	else:

		$is_archived = is_callable( [ $wishlist, 'is_archived' ] ) ? $wishlist->is_archived() : false;
		?>

		<fieldset <?php echo $is_archived ? 'disabled' : ''; ?>>

			<?php
			do_action( 'nmgr_before_messages', $wishlist, $messages_args );

			foreach ( $messages as $message ) :
				?>

				<div class="nmgr-message" data-message_id="<?php echo absint( $message->id ); ?>">
					<div class="icon"></div>
					<div class="nmgr-wrap">
						<div class="header">
							<div class="info">
								<span class="name"><?php echo esc_html( $message->name ); ?></span>
								<i><?php esc_html_e( 'on', 'nm-gift-registry' ); ?></i>
								<span class="date-created underline"><?php echo esc_html( $message->date_created ); ?></span>
								<i><?php esc_html_e( 'with', 'nm-gift-registry' ); ?></i>
								<span class="order-number underline">
									<?php
									printf(
										/* translators: %d: order number */
										__( 'Order #%d', 'nm-gift-registry' ),
										$message->order_id
									);
									?>
								</span>
							</div>

							<?php if ( !$is_archived ) : ?>
								<a href="#" class="nmgr-tip delete-wishlist-message"
									 title="<?php esc_attr_e( 'Delete', 'nm-gift-registry' ); ?>"
									 data-nmgr_post_action="delete_message"
									 data-notice="<?php esc_attr_e( 'Are you sure you want to delete this message?', 'nm-gift-registry' ); ?>"
									 data-message_id="<?php echo absint( $message->id ); ?>"
									 data-wishlist_id="<?php echo absint( $wishlist->get_id() ); ?>">
										 <?php
										 echo nmgr_get_svg( array( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
											 'icon' => 'trash-can',
											 'fill' => '#ccc'
										 ) );
										 ?>
								</a>
							<?php endif; ?>

						</div>
						<div class="content"><?php echo esc_html( $message->content ); ?></div>
						<?php if ( isset( $message->items_ordered ) && !empty( $message->items_ordered ) ) : ?>
							<div class="footer">
								<span class="items-ordered-title title"><strong><?php esc_html_e( 'Items ordered:', 'nm-gift-registry' ); ?></strong></span>
								<div class="items-ordered-content content">
									<?php
									$string = '';
									foreach ( $message->items_ordered as $item ) {
										$string .= "{$item[ 'name' ]} &times; {$item[ 'quantity' ]}, ";
									}
									echo esc_html( rtrim( trim( $string ), ',' ) );
									?>
								</div>
							</div>
						<?php endif; ?>
						<?php do_action( 'nmgr_message_footer', $message ); ?>
					</div>
				</div>

				<?php
			endforeach;

			do_action( 'nmgr_after_messages', $wishlist, $messages_args );
			?>

		</fieldset>

	<?php
	endif;
	?>

</div>


