<?php
/**
 * Wishlist orders
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.1.0
 */
defined( 'ABSPATH' ) || exit;
?>

<div <?php echo nmgr_format_attributes( $attributes ); ?>>

	<?php
	if ( !$wishlist || empty( $orders ) ):

		if ( !$wishlist ) :
			echo nmgr_get_default_account_section_content( 'orders', $wishlist );
		else :
			if ( empty( $orders ) && has_action( 'nmgr_no_orders' ) ) :
				do_action_deprecated( 'nmgr_no_orders', array( $wishlist ), '2.2.1', '', __( 'Use the filters and actions in \'nmgr_get_default_account_section_content\' instead', 'nm-gift-registry' ) );
			elseif ( empty( $orders ) ) :
				echo nmgr_get_default_account_section_content( 'orders', $wishlist );
			endif;
		endif;

	else:

		do_action( 'nmgr_before_orders', $wishlist );
		?>
		<table class="nmgr-orders-table nmgr-table">
			<thead>
				<tr>
					<?php
					foreach ( $order_columns as $key => $column ) {
						$column_label = $column[ 'label' ] ?? $column;
						$column_title = $column[ 'title' ] ?? '';
						echo '<th data-tiptip-position="top" class="nmgr-tip ' . esc_attr( $key ) . '" title="' . esc_attr( $column_title ) . '">' . esc_html( $column_label ) . '</th>';
					}
					?>
				</tr>
			</thead>
			<tbody>
				<?php foreach ( $orders as $order ) : ?>
					<tr>
						<?php
						$user = $order->get_user();
						$using_billing_details = false;
						$billing_full_name = trim( $order->get_formatted_billing_full_name() );

						if ( $user ) {
							$username = "$user->first_name $user->last_name";
							$customer = $username ? $username : $user->display_name;
							$email = $user->user_email;
						} else {
							$customer = $billing_full_name ? $billing_full_name : __( 'Guest', 'nm-gift-registry' );
							$email = $order->get_billing_email() ? $order->get_billing_email() : '';
							$using_billing_details = true;
						}

						foreach ( $order_columns as $key => $column ) :
							$column_label = $column[ 'label' ] ?? $column;
							?>
							<?php
							switch ( $key ) {
								case 'customer':
									echo '<td class="' . esc_attr( $key ) . '" data-title="' . esc_attr( $column_label ) . '" data-sort="' . esc_attr( $customer ) . '">';
									if ( !$billing_full_name && !$user ) {
										echo '<span class="nmgr-guest-text">' . esc_html( $customer ) . '</span>';
									} else {
										echo esc_html( $customer );
									}

									if ( $order->is_created_via( 'nmgr_wishlist' ) ) {
										?>
								<span class="nm badge" style="background-color:#ccc"
											data-bs-toggle="tooltip"
											data-bs-placement="right"
											title="<?php esc_attr_e( nmgr_get_custom_order_notice() ); ?>">&mcy;</span>
											<?php
										}

										if ( apply_filters( "nmgr_orders_table_{$key}_column_show_email", true ) && $email ) {
											echo '<div class="meta-item email">' .
											'<strong>' . __( 'Email:', 'nm-gift-registry' ) . ' </strong>' . sanitize_email( $email ) .
											'</div>';
										}

										if ( apply_filters( "nmgr_orders_table_{$key}_column_show_phone", true ) &&
											$using_billing_details && $order->get_billing_phone() ) {
											echo '<div class="meta-item phone"><strong>' . __( 'Tel:', 'nm-gift-registry' ) . ' </strong>' . esc_html( $order->get_billing_phone() ) . '</div>';
										}

										do_action( "nmgr_orders_table_{$key}_column" );

										echo '</td>';
										break;
									case 'order':
										echo '<td class="' . esc_attr( $key ) . '" data-title="' . esc_attr( $column_label ) . '" data-sort="' . $order->get_order_number() . '">';
										$label = sprintf(
											/* translators %s: order number */
											__( 'Order #%s', 'nm-gift-registry' ),
											esc_attr( $order->get_order_number() )
										);
										if ( $order->get_status() === 'trash' ) {
											echo $label;
										} else {
											echo '<a href="' . esc_url( get_edit_post_link( $order->get_id() ) ) . '">' . $label . '</a>';
										}
										do_action( "nmgr_orders_table_{$key}_column" );
										echo '</td>';
										break;
									case 'order-date':
										$datetime = new DateTime( $order->get_date_created() );
										echo '<td class="' . esc_attr( $key ) . '" data-title="' . esc_attr( $column_label ) . '" data-sort="' . $datetime->getTimestamp() . '">';
										echo nmgr_format_date( $order->get_date_created() );
										do_action( "nmgr_orders_table_{$key}_column" );
										echo '</td>';
										break;
									case 'items-purchased':
										echo '<td class="' . esc_attr( $key ) . '" data-title="' . esc_attr( $column_label ) . '"><ul class="items-purchased-list">';
										foreach ( $order->get_items() as $item_id => $item ) {
											$meta = $item->get_meta( 'nm_gift_registry' );
											if ( $meta && isset( $meta[ 'wishlist_id' ] ) && absint( $meta[ 'wishlist_id' ] ) === $wishlist->get_id() ) {
												echo '<li>';
												$product = $item->get_product();
												$permalink = null;
												if ( $product ) {
													$permalink = is_admin() ? get_edit_post_link( $product->get_id() ) : $product->get_permalink( $item->get_data() );
												}
												echo $permalink ? '<a href="' . esc_url( $permalink ) . '">' . wp_kses_post( $item->get_name() ) . '</a>' : wp_kses_post( $item->get_name() );
												echo ' &times; ' . ( int ) ($item->get_quantity() + $order->get_qty_refunded_for_item( $item_id ));
												echo '</li>';
											}
										}
										echo '</ul>';
										do_action( "nmgr_orders_table_{$key}_column" );
										echo '</td>';
										break;
									case 'total-spent':
										$total = nmgr_wishlist_get_amount_received_in_order( $wishlist->get_id(), $order->get_id() );
										echo '<td class="' . esc_attr( $key ) . '" data-title="' . esc_attr( $column_label ) . '" data-sort="' . $total . '">';
										echo wc_price( $total );
										do_action( "nmgr_orders_table_{$key}_column" );
										echo '</td>';
										break;
								}
								?>
							<?php endforeach; ?>
				</tr>
			<?php endforeach; ?>
			</tbody>
		</table>

		<?php
		do_action( 'nmgr_after_orders', $wishlist );

	endif;
	?>

</div>


