<?php
/**
 * Displays links for sharing wishlist on social networks
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.1.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;

$type = $wishlist->get_type();
$url = $wishlist->get_permalink();
$wishlist_title = $wishlist->get_title();
$desc = $wishlist->get_description() ?
	$wishlist->get_description() :
	apply_filters( 'nmgr_default_share_description', sprintf(
			/* translators: %s: wishlist type title */
			nmgr()->is_pro ? __( 'Here is the link to my %s:', 'nm-gift-registry' ) : __( 'Here is the link to my %s:', 'nm-gift-registry-lite' ),
			nmgr_get_type_title()
	) );
$image = '';

if ( method_exists( $wishlist, 'get_thumbnail_id' ) && $wishlist->get_thumbnail_id() ) {
	$image = wp_get_attachment_url( $wishlist->get_thumbnail_id() );
}
?>


<div class="nmgr-sharing">

	<?php
	$title = nmgr()->is_pro ?
		__( 'Share on:', 'nm-gift-registry' ) :
		__( 'Share on:', 'nm-gift-registry-lite' );
	printf( '<h3 class="nmgr-template-title sharing">%s</h3>', esc_html( $title ) );
	?>

	<div class="nmgr-sharing-options">
		<?php
		if ( nmgr_get_type_option( $type, 'share_on_facebook' ) ) :
			$share_link = add_query_arg(
				array(
					'u' => rawurlencode( $url ),
					'p[title]' => rawurlencode( $wishlist_title ),
				), 'http://www.facebook.com/sharer/sharer.php' );
			?>
			<div class="share-item nmgr-share-on-facebook">
				<a target="_blank" href="<?php echo esc_url( $share_link ); ?>" title="<?php
				echo esc_attr( nmgr()->is_pro ?
						__( 'Click to share on facebook', 'nm-gift-registry' ) :
						__( 'Click to share on facebook', 'nm-gift-registry-lite' )  );
				?>" class="nmgr-tip">
					 <?php
					 echo nmgr_get_svg( array( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						 'icon' => 'facebook',
						 'size' => 2.375,
						 'fill' => '#ddd'
					 ) );
					 ?>
				</a>
			</div>
		<?php endif; ?>

		<?php
		if ( nmgr_get_type_option( $type, 'share_on_twitter' ) ) :
			$share_link = add_query_arg(
				array(
					'url' => rawurlencode( $url ),
					'text' => rawurlencode( $desc ),
				), 'https://twitter.com/share' );
			?>
			<div class="share-item nmgr-share-on-twitter">
				<a target="_blank" href="<?php echo esc_url( $share_link ); ?>" title="<?php
				echo esc_attr( nmgr()->is_pro ?
						__( 'Click to share on twitter', 'nm-gift-registry' ) :
						__( 'Click to share on twitter', 'nm-gift-registry-lite' )  );
				?>" class="nmgr-tip">
					 <?php
					 echo nmgr_get_svg( array( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						 'icon' => 'twitter',
						 'size' => 2.375,
						 'fill' => '#ddd',
					 ) );
					 ?>
				</a>
			</div>
		<?php endif; ?>

		<?php
		if ( nmgr_get_type_option( $type, 'share_on_pinterest' ) ) :
			$share_link = add_query_arg(
				array(
					'url' => rawurlencode( $url ),
					'description' => rawurlencode( $desc ),
				), 'http://pinterest.com/pin/create/button/' );

			if ( $image ) :
				$share_link = add_query_arg( 'media', rawurlencode( $image ), $share_link );
			endif;
			?>
			<div class="share-item nmgr-share-on-pinterest">
				<a target="_blank" href="<?php echo esc_url( $share_link ); ?>" title="<?php
				echo esc_attr( nmgr()->is_pro ?
						__( 'Click to share on pinterest', 'nm-gift-registry' ) :
						__( 'Click to share on pinterest', 'nm-gift-registry-lite' )  );
				?>" class="nmgr-tip">
					 <?php
					 echo nmgr_get_svg( array( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						 'icon' => 'pinterest',
						 'size' => 2.375,
						 'fill' => '#ddd'
					 ) );
					 ?>
				</a>
			</div>
		<?php endif; ?>

		<?php
		if ( nmgr_get_type_option( $type, 'share_on_whatsapp' ) ) :
			$share_link = add_query_arg(
				array(
					'text' => rawurlencode( $wishlist_title . "\r\n\r\n" . $desc . "\r\n\r\n" . $url ),
				), 'https://wa.me/' );
			?>
			<div class="share-item nmgr-share-on-whatsapp">
				<a target="_blank" href="<?php echo $share_link; ?>" title="<?php
				echo esc_attr( nmgr()->is_pro ?
						__( 'Click to share on whatsapp', 'nm-gift-registry' ) :
						__( 'Click to share on whatsapp', 'nm-gift-registry-lite' )  );
				?>" class="nmgr-tip">
					 <?php
					 echo nmgr_get_svg( array(
						 'icon' => 'whatsapp',
						 'size' => 2.375,
						 'fill' => '#ddd'
					 ) );
					 ?>
				</a>
			</div>
		<?php endif; ?>

		<?php
		if ( nmgr_get_type_option( $type, 'share_on_email' ) ) :
			$share_link = add_query_arg(
				array(
					'subject' => rawurlencode( apply_filters( 'nmgr_share_on_email_subject', $wishlist_title ) ),
					'body' => rawurlencode( apply_filters( 'nmgr_share_on_email_body', $desc . ' ' . $url ) ),
					'title' => rawurlencode( $wishlist_title ),
				), 'mailto:' );
			?>
			<div class="share-item nmgr-share-on-email">
				<a target="_blank" href="<?php echo esc_url( $share_link ); ?>" title="<?php
				echo esc_attr( nmgr()->is_pro ?
						__( 'Click to share via email', 'nm-gift-registry' ) :
						__( 'Click to share via email', 'nm-gift-registry-lite' )  );
				?>" class="nmgr-tip">
					<?php
					echo nmgr_get_svg( array( // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						'icon' => 'email',
						'size' => 2.375,
						'fill' => '#ddd'
					) );
					?>
				</a>
			</div>
		<?php endif; ?>

	</div>
</div>