<?php
/**
 * Template for listing all wishlists for a user on the account page
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.0.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;

do_action( 'nmgr_before_account_wishlists', $wishlists );

$type = nmgr_get_current_type();

$columns = [
	'title' => [
		'label' => nmgr()->is_pro ?
		__( 'Title', 'nm-gift-registry' ) :
		__( 'Title', 'nm-gift-registry-lite' ),
	],
	'created' => [
		'label' => nmgr()->is_pro ?
		__( 'Created', 'nm-gift-registry' ) :
		__( 'Created', 'nm-gift-registry-lite' ),
	],
	'visibility' => [
		'label' => nmgr()->is_pro ?
		__( 'Visibility', 'nm-gift-registry' ) :
		__( 'Visibility', 'nm-gift-registry-lite' ),
	],
	'items' => [
		'label' => nmgr()->is_pro ?
		__( 'Items', 'nm-gift-registry' ) :
		__( 'Items', 'nm-gift-registry-lite' ),
	],
	'total' => [
		'label' => nmgr()->is_pro ?
		__( 'Total', 'nm-gift-registry' ) :
		__( 'Total', 'nm-gift-registry-lite' ),
	],
	'actions' => [
		'label' => nmgr()->is_pro ?
		__( 'Actions', 'nm-gift-registry' ) :
		__( 'Actions', 'nm-gift-registry-lite' ),
	],
];

if ( !nmgr()->is_pro ) {
	unset( $columns[ 'visibility' ] );
}
?>

<div class="nmgr-account-wishlists-header">
	<div class="count">
		<?php
		$no_of_wishlists = is_object( $wishlists ) ? 1 : count( $wishlists );
		/* translators: 1: wishlists count, 2: wishlists type title */
		printf( nmgr()->is_pro ? _n( '%1$s  %2$s', '%1$s %2$s', $no_of_wishlists, 'nm-gift-registry' ) : _n( '%1$s  %2$s', '%1$s %2$s', $no_of_wishlists, 'nm-gift-registry-lite' ),
			'<strong>' . number_format_i18n( $no_of_wishlists ) . '</strong>',
			1 === $no_of_wishlists ?
				esc_html( nmgr_get_type_title( '', '', $type ) ) :
				esc_html( nmgr_get_type_title( '', true, $type ) )
		);
		?>
	</div>
</div>

<table class="nmgr-table nmgr-wishlists-table">
	<thead>
		<tr>
			<?php foreach ( $columns as $column ) : ?>
				<th><?php echo esc_html( $column[ 'label' ] ); ?></th>
			<?php endforeach; ?>
		</tr>
	</thead>
	<tbody>
		<?php
		foreach ( $wishlists as $wishlist ) :
			?>
			<tr>
				<?php foreach ( $columns as $key => $column ) : ?>
					<td data-title="<?php echo esc_html( $column[ 'label' ] ); ?>">
						<?php
						switch ( $key ) {
							case 'title':
								?>
								<a href="<?php echo esc_url( $wishlist->get_permalink() ); ?>">
									<?php echo esc_html( $wishlist->get_title() ); ?>
								</a>
								<?php
								break;
							case 'created':
								echo wp_kses_post( nmgr_format_date( $wishlist->get_date_created() ) );
								break;
							case 'visibility':
								if ( method_exists( $wishlist, 'get_visibility' ) ) {
									echo esc_html( ucfirst( $wishlist->get_visibility() ) );
								}
								break;
							case 'items':
								echo absint( $wishlist->get_items_quantity_count() );
								break;
							case 'total':
								echo wp_kses_post( $wishlist->get_total( true ) );
								break;
							case 'actions':
								$dropdown_actions = [
									'view' => [
										'text' => nmgr()->is_pro ?
										__( 'View', 'nm-gift-registry' ) :
										__( 'View', 'nm-gift-registry-lite' ),
										'attributes' => [
											'class' => [
												'wishlists-action',
												'view'
											],
											'href' => $wishlist->get_permalink(),
										],
									],
								];

								$dropdown = nmgr_get_dropdown();

								foreach ( $dropdown_actions as $dkey => $daction ) {
									$dropdown->set_menu_item(
										($daction[ 'text' ] ?? '' ),
										($daction[ 'attributes' ] ?? [] ),
										$dkey
									);
								}

								$dropdown->print();
								break;
						}
						?>
					</td>
				<?php endforeach; ?>
			</tr>
		<?php endforeach; ?>
	</tbody>
</table>


<?php
do_action( 'nmgr_after_account_wishlists', $wishlists );

