<?php
/**
 * Template for displaying the add to wishlist view in a dialog
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.0.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;

if ( !isset( $product ) ) {
	$product = $GLOBALS[ 'product' ];
}
$product_id = absint( $product->get_id() );
?>

<form class="nmgr-add-to-wishlist-content product-type-<?php echo esc_attr( $product->get_type() ); ?>">
	<?php
	do_action( 'nmgr_add_to_wishlist_dialog_content_before', $args );

	$thumbnail = $product->get_image( 'nmgr_thumbnail', array(
		'title' => $product->get_name(),
		'alt' => $product->get_name() ) );
	?>
	<div class="product-thumbnail">
		<?php echo nmgr_kses_post( $thumbnail ); ?>
	</div>
	<div class="product-title nmgr-row">
		<?php echo esc_html( $product->get_name() ); ?>
		<span class="product-price">(<?php echo $product->get_price_html(); ?>)</span>
	</div>

	<?php do_action( 'nmgr_add_to_wishlist_dialog_content_after_title', $args ); ?>

	<?php if ( !empty( $wishlists ) ) : ?>
		<div class="nmgr-row">
			<label class="nmgr-select-wishlist-label">
				<span class="nmgr-text">
					<?php
					printf(
						/* translators %s: wishlist type title */
						esc_html( nmgr()->is_pro ? __( 'Select %s', 'nm-gift-registry' ) : __( 'Select %s', 'nm-gift-registry-lite' )  ),
						esc_html( nmgr_get_type_title( '', false, $args[ 'type' ] ) )
					);
					?>
				</span>
				<select name="nmgr_wid" class="list-of-wishlists">
					<?php
					foreach ( $wishlists as $wishlist ) {
						$wishlist_id = absint( $wishlist->get_id() );
						$shipping_address_required = $wishlist->needs_shipping_address() ? 1 : 0;
						$title = !$wishlist->has_product( $product ) ? '' : sprintf(
								/* translators: %s: wishlist type title */
								nmgr()->is_pro ? __( 'This product is in this %s', 'nm-gift-registry' ) : __( 'This product is in this %s', 'nm-gift-registry-lite' ),
								nmgr_get_type_title( '', false, $args[ 'type' ] )
						);
						?>
						<option value="<?php echo $wishlist_id; ?>"
										data-shipping-address-required="<?php echo $shipping_address_required; ?>"
										<?php
										if ( isset( $selected_wishlist_id ) && absint( $selected_wishlist_id ) === $wishlist_id ) {
											echo 'selected="selected"';
										}
										?>
										title="<?php echo esc_attr( $title ); ?>">
											<?php
											echo esc_html( $wishlist->get_title() );

											echo $wishlist->has_product( $product ) ? '  &hearts;' : '';
											?>
						</option>
					<?php } ?>
				</select>
			</label>
		</div>
		<?php
		do_action( 'nmgr_add_to_wishlist_dialog_content_after_wishlists', $args );

		if ( $product->is_type( 'grouped' ) ) :
			nmgr_template( 'add-to-wishlist/options-grouped.php', $args );
		elseif ( nmgr()->is_pro && 'gift-registry' === $args[ 'type' ] ) :
			nmgr_template( 'add-to-wishlist/options-simple-variable.php', $args );
		endif;

	else:
		?>
		<div class="nmgr-atw-empty-wishlists-message" style="color:#999;margin-bottom:11px;">
			<?php
			printf(
				/* translators: %s: wishlist type title */
				esc_html( nmgr()->is_pro ? __( 'You have no active %s to add this product to.', 'nm-gift-registry' ) : __( 'You have no active %s to add this product to.', 'nm-gift-registry-lite' )  ),
				esc_html( nmgr_get_type_title( '', true, $args[ 'type' ] ) )
			);
			?>
		</div>
	<?php
	endif; // !empty($wishlists)

	do_action_deprecated( 'nmgr_add_to_wishlist_dialog_content_after_options', [ $args ], '3.0.0', 'nmgr_add_to_wishlist_dialog_content_after' );

	do_action( 'nmgr_add_to_wishlist_dialog_content_after', $args );

	foreach ( $formdata as $key => $value ) {
		echo '<input type="hidden" name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '">';
	}
	?>
</form><!-- .add-to-wishlist-content -->
