<?php
/**
 * Template for displaying the options for adding grouped products to the wishlist
 * in the add-to-wishlist dialog
 *
 * @link https://docs.nmerimedia.com/nm-gift-registry-and-wishlist/overriding-templates/
 * @version 4.0.0
 * @sync
 */
defined( 'ABSPATH' ) || exit;

if ( isset( $grouped_products ) && !empty( $grouped_products ) ) :
	foreach ( $grouped_products as $grouped_product ) :
		if ( $grouped_product->is_purchasable() &&
			!$grouped_product->has_options() &&
			$grouped_product->is_in_stock() ) :
			?>
			<div class="nmgr-row option-row qty-fav">
				<?php
				do_action( 'nmgr_add_to_wishlist_option_row_start', $grouped_product, $args );

				if ( nmgr()->is_pro && 'gift-registry' === $args[ 'type' ] &&
					nmgr_get_option( 'display_item_quantity' ) &&
					isset( $wishlist_item_options[ 'qty' ][ $grouped_product->get_id() ] ) ) :
					$data_title_on = sprintf(
						/* translators: %s: wishlist type title */
						__( 'Quantity already in this %s', 'nm-gift-registry' ),
						nmgr_get_type_title()
					);
					?>

					<div class="nmgr-cart-qty-wrapper nmgr-hide"
							 title="<?php echo $data_title_on; ?>"
							 <?php echo $wishlist_item_options[ 'qty' ][ $grouped_product->get_id() ]; ?>>
						<div class="nmgr-cart-qty">
							<?php
							echo nmgr_get_svg( array(
								'icon' => 'cart-empty',
								'size' => 1.6,
								'fill' => '#bbb',
							) );
							?>
							<span class="nmgr-qty"></span>
						</div>
					</div>
					<?php
				endif;

				if ( nmgr()->is_pro ) :
					if ( 'gift-registry' == $args[ 'type' ] && nmgr_get_option( 'display_item_quantity', 1 ) ) :
						woocommerce_quantity_input(
							array(
								'input_name' => nmgr_query_key( 'quantity' ) . '[' . $grouped_product->get_id() . ']',
								'input_value' => 0,
								'min_value' => apply_filters( 'nmgr_quantity_input_min', 0, $grouped_product ),
								'max_value' => apply_filters( 'nmgr_quantity_input_max', $grouped_product->get_stock_quantity(), $grouped_product ),
							),
							$grouped_product
						);
					endif;
				else :
					$quantity = nmgr_query_key( 'quantity' );
					$field_id = uniqid( "{$quantity}_{$grouped_product->get_id()}_" );
					?>
					<div class="nmgr-btn-group">
						<div class="nmgr-btn">
							<input type="checkbox"
										 value="1"
										 id="<?php echo esc_attr( $field_id ); ?>"
										 name="nmgr_qty[<?php echo esc_attr( $grouped_product->get_id() ); ?>]">
							<label for="<?php echo esc_attr( $field_id ); ?>"
										 class="title"
										 title="<?php
										 printf(
											 /* translators: %s: wishlist type title */
											 esc_html__( 'Add this item to my %s', 'nm-gift-registry-lite' ), esc_html( nmgr_get_type_title() ) );
										 ?>">
											 <?php echo esc_html( $grouped_product->get_name() ); ?>
							</label>
						</div>
					</div>
				<?php
				endif;

				if ( nmgr()->is_pro && 'gift-registry' == $args[ 'type' ] && nmgr_get_option( 'display_item_favourite' ) ) :
					?>
					<div class="favourite">
						<?php
						$field_id = esc_attr( uniqid( "nmgr_fav_{$grouped_product->get_id()}_" ) );
						$data_title_off = __( 'Mark as favourite', 'nm-gift-registry' );
						$data_title_on = sprintf(
							/* translators: %s: wishlist type title */
							esc_attr__( 'This item is a favourite in this %s', 'nm-gift-registry' ), nmgr_get_type_title()
						);
						?>
						<div class="nmgr-btn-group">
							<div class="nmgr-btn">
								<input id="<?php echo $field_id; ?>" type="checkbox" value="1"
											 name="nmgr_fav<?php echo '[' . $grouped_product->get_id() . ']'; ?>"
											 <?php
											 if ( isset( $wishlist_item_options[ 'nmgr_fav' ][ $grouped_product->get_id() ] ) ) {
												 echo $wishlist_item_options[ 'nmgr_fav' ][ $grouped_product->get_id() ];
											 }
											 ?>>
								<label for="<?php echo $field_id; ?>"
											 class="icon"
											 data-title-on="<?php echo esc_attr( $data_title_on ); ?>"
											 data-title-off="<?php echo esc_attr( $data_title_off ); ?>"
											 title="<?php echo esc_attr( $data_title_off ); ?>">
									<span class='nmgr-icon'>&bigstar;</span>
								</label>
							</div>
						</div>
					</div>
					<?php
				endif;

				do_action( 'nmgr_add_to_wishlist_option_row_end', $grouped_product, $args );

				if ( nmgr()->is_pro && ('wishlist' == $args[ 'type' ] ||
					('gift-registry' == $args[ 'type' ] && !nmgr_get_option( 'display_item_quantity' ))) ) :

					$field_id = uniqid( "nmgr_qty_{$grouped_product->get_id()}_" );
					?>
					<div class="nmgr-btn-group">
						<div class="nmgr-btn">
							<input type="checkbox"
										 value="1"
										 id="<?php echo esc_attr( $field_id ); ?>"
										 name="nmgr_qty[<?php echo esc_attr( $grouped_product->get_id() ); ?>]">
							<label for="<?php echo esc_attr( $field_id ); ?>"
										 class="title"
										 title="<?php
										 printf(
											 /* translators: %s: wishlist type title */
											 esc_html__( 'Add this item to my %s', 'nm-gift-registry' ), esc_html( nmgr_get_type_title() ) );
										 ?>">
											 <?php echo esc_html( $grouped_product->get_name() ) . '  <span class="product-price">(' . $grouped_product->get_price_html() . ')</span>'; ?>
							</label>
						</div>
					</div>

					<?php
				elseif ( nmgr()->is_pro && 'gift-registry' === $args[ 'type' ] && nmgr_get_option( 'display_item_quantity' ) ):
					?>

					<div class="title"><?php echo esc_html( $grouped_product->get_name() ) . ' <span class="product-price">(' . $grouped_product->get_price_html() . ')</span>'; ?></div>

					<?php
				endif;
				?>

			</div>
			<?php
		endif;
	endforeach;
endif;